#-*- coding: utf-8 -*-

from django.db.models import Manager
from django.shortcuts import get_object_or_404
from django.db.models import Q


class TopicPrivateManager(Manager):

    def for_delete_or_404(self, pk, user):
        # User is the creator or wants to leave
        return get_object_or_404(self,
                                 Q(topic__user=user) | Q(user=user),
                                 pk=pk)

    def for_create_or_404(self, topic_id, user):
        # User is creator and has access
        return get_object_or_404(self,
                                 topic_id=topic_id,
                                 user=user,
                                 topic__user=user)


#-*- coding: utf-8 -*-

from .tags import comment
from .tags import comment_bookmark
from .tags import comment_like
from .tags import topic_poll
from .tags import search
from .tags import topic_favorite
from .tags import topic_notification
from .tags import topic_private
from .tags.utils import gravatar
from .tags.utils import messages
from .tags.utils import paginator
from .tags.utils import social_share
from .tags.utils import time

from .tags import register


#-*- coding: utf-8 -*-

# DO NOT EDIT THIS FILE
# YOU MAY EXTEND/OVERWRITE THE DEFAULT VALUES IN YOUR settings.py FILE

import os

ST_COMMENTS_PER_PAGE = 20
ST_COMMENTS_PAGE_VAR = 'page'

ST_TOPIC_PRIVATE_CATEGORY_PK = 1
ST_UNCATEGORIZED_CATEGORY_PK = 2

ST_RATELIMIT_ENABLE = True
ST_RATELIMIT_CACHE_PREFIX = 'srl'
ST_RATELIMIT_CACHE = 'default'

ST_NOTIFICATIONS_PER_PAGE = 20

ST_MENTIONS_PER_COMMENT = 30

ST_YT_PAGINATOR_PAGE_RANGE = 3

ST_SEARCH_QUERY_MIN_LEN = 3

ST_USER_LAST_SEEN_THRESHOLD_MINUTES = 1

ST_PRIVATE_FORUM = False

ST_ALLOWED_UPLOAD_IMAGE_FORMAT = ('jpeg', 'png', 'gif')

# check out http://pythonhosted.org/Markdown/extensions/index.html
ST_MARKDOWN_EXT = (
    'nl2br',
    'fenced_code',
    'spirit.utils.markdown.mention',
    'spirit.utils.markdown.emoji',
    'spirit.utils.markdown.image',
    'spirit.utils.markdown.video',
    'spirit.utils.markdown.audio',
    'spirit.utils.markdown.youtube',
    'spirit.utils.markdown.vimeo',
)


#
# Django & Spirit settings defined below...
#

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'spirit',
    #'debug_toolbar'
)

# python manage.py createcachetable spirit_cache
CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.db.DatabaseCache',
        'LOCATION': 'spirit_cache',
    },
}

AUTH_USER_MODEL = 'spirit.User'

AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',
    'spirit.backends.user.EmailAuthBackend',
)

LOGIN_URL = 'spirit:user-login'
LOGIN_REDIRECT_URL = 'spirit:profile-update'

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    #'spirit.middleware.XForwardedForMiddleware',
    'spirit.middleware.TimezoneMiddleware',
    'spirit.middleware.LastIPMiddleware',
    'spirit.middleware.LastSeenMiddleware',
    'spirit.middleware.ActiveUserMiddleware',
    'spirit.middleware.PrivateForumMiddleware',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    'django.core.context_processors.static',
    'django.core.context_processors.tz',
    'django.core.context_processors.request',
    'django.contrib.messages.context_processors.messages',
)

# Keep templates in memory
TEMPLATE_LOADERS = (
    ('django.template.loaders.cached.Loader', (
        'django.template.loaders.filesystem.Loader',
        'django.template.loaders.app_directories.Loader',
    )),
)

#
# Third-party apps settings defined below...
#

# django-djconfig

INSTALLED_APPS += (
    'djconfig',
)

DJC_BACKEND = 'djconfig'

CACHES.update({
    'djconfig': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
    },
})

MIDDLEWARE_CLASSES += (
    'djconfig.middleware.DjConfigLocMemMiddleware',
)

TEMPLATE_CONTEXT_PROCESSORS += (
    'djconfig.context_processors.config',
)

# django-haystack

INSTALLED_APPS += (
    'haystack',
)

HAYSTACK_CONNECTIONS = {
    'default': {
        'ENGINE': 'haystack.backends.whoosh_backend.WhooshEngine',
        'PATH': os.path.join(os.path.dirname(__file__), 'whoosh_index'),
    },
}


#-*- coding: utf-8 -*-

from django.conf.urls import patterns, url


urlpatterns = patterns("spirit.views.admin.topic",
    url(r'^$', 'topic_deleted', name='admin-topic'),
    url(r'^deleted/$', 'topic_deleted', name='admin-topic-deleted'),
    url(r'^closed/$', 'topic_closed', name='admin-topic-closed'),
    url(r'^pinned/$', 'topic_pinned', name='admin-topic-pinned'),
)


#-*- coding: utf-8 -*-

from django.conf.urls import patterns, url


urlpatterns = patterns("spirit.views.comment_bookmark",
    url(r'^(?P<topic_id>\d+)/create/$', 'bookmark_create', name='bookmark-create'),
    url(r'^(?P<topic_id>\d+)/find/$', 'bookmark_find', name='bookmark-find'),
)


#-*- coding: utf-8 -*-

import os
import sys

os.environ['DJANGO_SETTINGS_MODULE'] = 'example.settings_test_runner'

from django.test.runner import DiscoverRunner


def run_tests():
    test_runner = DiscoverRunner()
    failures = test_runner.run_tests(["spirit", ])
    sys.exit(failures)


if __name__ == "__main__":
    run_tests()


#-*- coding: utf-8 -*-

import urllib


def get_page_number(obj_number, per_page):
    if obj_number < per_page:
        return 1
    elif obj_number % per_page:
        return obj_number / per_page + 1
    else:
        return obj_number / per_page


def get_url(url, obj_number, per_page, page_var):
    page = get_page_number(obj_number, per_page)
    data = urllib.urlencode({page_var: page, })

    if page == 1:
        return "".join((url, '#c', str(obj_number)))

    return "".join((url, '?', data, '#c', str(obj_number)))


#-*- coding: utf-8 -*-

from django.views.generic import ListView
from django.shortcuts import render, get_object_or_404
from django.http import HttpResponsePermanentRedirect

from spirit.models.topic import Topic

from spirit.models.category import Category


def category_detail(request, pk, slug):
    category = Category.objects.get_public_or_404(pk=pk)

    if category.slug != slug:
        return HttpResponsePermanentRedirect(category.get_absolute_url())

    subcategories = Category.objects.for_parent(parent=category)
    topics = Topic.objects.for_category(category=category)\
        .order_by('-is_pinned', '-last_active')\
        .select_related('category')

    return render(request, 'spirit/category/category_detail.html', {'category': category,
                                                                    'subcategories': subcategories,
                                                                    'topics': topics})


class CategoryList(ListView):

    template_name = 'spirit/category/category_list.html'
    context_object_name = "categories"
    queryset = Category.objects.for_parent()


"""
Django settings for test2 project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/
"""

# You may override spirit settings below...

from spirit.settings import *


# Build paths inside the project like this: os.path.join(BASE_DIR, ...)

import os
BASE_DIR = os.path.dirname(os.path.dirname(__file__))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'change-me'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = False

TEMPLATE_DEBUG = False

ALLOWED_HOSTS = []


# Application definition

# Extend the Spirit installed apps (notice the plus sign)
# Check out the spirit.settings.py so you do not end up with duplicate apps.
INSTALLED_APPS += (
    #'my_app1',
    #'my_app2',
)

# same here, check out the spirit.settings.py
MIDDLEWARE_CLASSES += (
    #'my_middleware1',
    #'my_middleware2',
)

# same here
TEMPLATE_CONTEXT_PROCESSORS += (
    #'my_template_proc1',
    #'my_template_proc2',
)

# same here (we update the Spirit caches)
CACHES.update({
    #'default': {
    #  'BACKEND': 'my.backend.path',
    #},
})


ROOT_URLCONF = 'example.urls'

WSGI_APPLICATION = 'example.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = 'en'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/

STATIC_URL = '/static/'

MEDIA_URL = '/media/'

MEDIA_ROOT = os.path.join(BASE_DIR, 'media')

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error when DEBUG=False.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}

try:
    # devs must create this file to override settings
    # local_settings_sample.py is provided
    from local_settings import *
except ImportError:
    pass


#-*- coding: utf-8 -*-

from django import forms
from django.utils.translation import ugettext_lazy as _

from ..utils.forms import NestedModelChoiceField
from spirit.models.category import Category

from spirit.models.topic import Topic


class TopicForm(forms.ModelForm):

    class Meta:
        model = Topic
        fields = ('title', 'category')

    def __init__(self, user, *args, **kwargs):
        super(TopicForm, self).__init__(*args, **kwargs)
        self.user = user
        # TODO: add custom Prefetch object to filter closed sub-categories, on Django 1.7
        self.fields['category'] = NestedModelChoiceField(queryset=Category.objects.for_public_open(),
                                                         related_name='category_set',
                                                         parent_field='parent_id',
                                                         label_field='title',
                                                         label=_("Category"),
                                                         empty_label=_("Chose a category"))

        if self.instance.pk and not user.is_moderator:
            del self.fields['category']

    def clean_category(self):
        # TODO: remove this on django 1.7
        category = self.cleaned_data['category']

        if category.is_closed or category.is_removed:
            raise forms.ValidationError(_("The chosen category is closed"))

        return category

    def save(self, commit=True):
        if not self.instance.pk:
            self.instance.user = self.user

        return super(TopicForm, self).save(commit)


#-*- coding: utf-8 -*-

from django import forms
from django.utils.html import conditional_escape, mark_safe
from django.utils.encoding import smart_text


class NestedModelChoiceField(forms.ModelChoiceField):
    """A ModelChoiceField that groups parents and childrens"""
    # TODO: subclass ModelChoiceIterator, remove _populate_choices()
    def __init__(self, related_name, parent_field, label_field, *args, **kwargs):
        """
        @related_name: related_name or "FOO_set"
        @parent_field: ForeignKey('self') field, use 'name_id' to save some queries
        @label_field: field for obj representation
        """
        super(NestedModelChoiceField, self).__init__(*args, **kwargs)
        self.related_name = related_name
        self.parent_field = parent_field
        self.label_field = label_field
        self._populate_choices()

    def _populate_choices(self):
        # This is *hackish* but simpler than subclassing ModelChoiceIterator
        choices = [("", self.empty_label), ]
        kwargs = {self.parent_field: None, }
        queryset = self.queryset.filter(**kwargs)\
            .prefetch_related(self.related_name)

        for parent in queryset:
            choices.append((self.prepare_value(parent), self.label_from_instance(parent)))
            choices.extend([(self.prepare_value(children), self.label_from_instance(children))
                            for children in getattr(parent, self.related_name).all()])

        self.choices = choices

    def label_from_instance(self, obj):
        level_indicator = u""

        if getattr(obj, self.parent_field):
            level_indicator = u"--- "

        return mark_safe(level_indicator + conditional_escape(smart_text(getattr(obj, self.label_field))))


#-*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from django.conf import settings


class TopicPoll(models.Model):

    topic = models.OneToOneField('spirit.Topic', verbose_name=_("topic"), primary_key=True, related_name='poll')

    date = models.DateTimeField(auto_now_add=True)
    choice_limit = models.PositiveIntegerField(_("choice limit"), default=1)
    is_closed = models.BooleanField(default=False)

    class Meta:
        app_label = 'spirit'
        verbose_name = _("topic poll")
        verbose_name_plural = _("topics polls")

    def get_absolute_url(self):
        return self.topic.get_absolute_url()

    def __unicode__(self):
        return "poll at topic #%s" % self.topic.pk


class TopicPollChoice(models.Model):

    poll = models.ForeignKey(TopicPoll, verbose_name=_("poll"), related_name='choices')

    description = models.CharField(_("choice description"), max_length=255)
    vote_count = models.PositiveIntegerField(_("vote count"), default=0)

    class Meta:
        app_label = 'spirit'
        verbose_name = _("poll choice")
        verbose_name_plural = _("poll choices")

    def __unicode__(self):
        return "poll choice %s at topic #%s" % (self.pk, self.poll.topic.pk)


class TopicPollVote(models.Model):

    choice = models.ForeignKey(TopicPollChoice, verbose_name=_("poll choice"), related_name='votes')
    user = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_("voter"))

    date = models.DateTimeField(auto_now_add=True)

    class Meta:
        app_label = 'spirit'
        unique_together = ('user', 'choice')
        verbose_name = _("poll vote")
        verbose_name_plural = _("poll votes")

    def __unicode__(self):
        return "poll vote %s" % self.pk


#-*- coding: utf-8 -*-

from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse

from spirit.managers.topic_private import TopicPrivateManager


class TopicPrivate(models.Model):

    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    topic = models.ForeignKey('spirit.Topic', related_name='topics_private')

    date = models.DateTimeField(auto_now_add=True)

    objects = TopicPrivateManager()

    class Meta:
        app_label = 'spirit'
        unique_together = ('user', 'topic')
        ordering = ['-date', ]
        verbose_name = _("private topic")
        verbose_name_plural = _("private topics")

    def get_absolute_url(self):
        return self.topic.get_absolute_url()

    def __unicode__(self):
        return "%s participes in %s" % (self.user, self.topic)


#-*- coding: utf-8 -*-

from . import register
from spirit.forms.topic_private import TopicPrivateInviteForm


@register.inclusion_tag('spirit/topic_private/_private_invite_form.html')
def render_invite_form(topic, next=None):
    form = TopicPrivateInviteForm()
    return {'form': form, 'topic': topic, 'next': next}


#-*- coding: utf-8 -*-

from django.contrib.auth.decorators import login_required
from django.shortcuts import render, redirect, get_object_or_404
from django.core.urlresolvers import reverse
from django.contrib.auth import get_user_model, login
from django.contrib.auth.views import login as login_view
from django.contrib.auth.views import password_reset, logout
from django.contrib.auth.forms import PasswordChangeForm
from django.contrib import messages
from django.utils.translation import ugettext as _
from django.http import HttpResponsePermanentRedirect

from spirit.utils.ratelimit.decorators import ratelimit
from spirit.utils.user.email import send_activation_email, send_email_change_email
from spirit.utils.user.tokens import UserActivationTokenGenerator, UserEmailChangeTokenGenerator

from ..models.topic import Topic
from ..models.comment import Comment

from ..forms.user import UserProfileForm, RegistrationForm, LoginForm, EmailChangeForm, ResendActivationForm


User = get_user_model()


@ratelimit(field='username', rate='5/5m')
def custom_login(request, **kwargs):
    # Current Django 1.5 login view does not redirect somewhere if the user is logged in
    if request.user.is_authenticated():
        return redirect(request.GET.get('next', request.user.get_absolute_url()))

    if request.is_limited and request.method == "POST":
        return redirect(request.get_full_path())

    return login_view(request, authentication_form=LoginForm, **kwargs)


def custom_logout(request, **kwargs):
    # Current Django 1.6 uses GET to log out
    if not request.user.is_authenticated():
        return redirect(request.GET.get('next', reverse('spirit:user-login')))

    if request.method == 'POST':
        return logout(request, **kwargs)

    return render(request, 'spirit/user/logout.html')


@ratelimit(field='email', rate='5/5m')
def custom_reset_password(request, **kwargs):
    if request.is_limited and request.method == "POST":
        return redirect(reverse("spirit:password-reset"))

    return password_reset(request, **kwargs)


@ratelimit(rate='2/10s')
def register(request):
    if request.user.is_authenticated():
        return redirect(request.GET.get('next', reverse('spirit:profile-update')))

    if request.method == 'POST':
        form = RegistrationForm(data=request.POST)

        if not request.is_limited and form.is_valid():
            user = form.save()
            send_activation_email(request, user)
            messages.info(request, _("We have sent you an email so you can activate your account!"))

            # TODO: email-less activation
            #if not settings.REGISTER_EMAIL_ACTIVATION_REQUIRED:
                #login(request, user)
                #return redirect(request.GET.get('next', reverse('spirit:profile-update')))

            return redirect(reverse('spirit:user-login'))
    else:
        form = RegistrationForm()

    return render(request, 'spirit/user/register.html', {'form': form, })


def registration_activation(request, pk, token):
    user = get_object_or_404(User, pk=pk)
    activation = UserActivationTokenGenerator()

    if activation.is_valid(user, token):
        user.is_active = True
        user.save()
        messages.info(request, _("Your account has been activated!"))

    return redirect(reverse('spirit:user-login'))


@ratelimit(field='email', rate='5/5m')
def resend_activation_email(request):
    if request.user.is_authenticated():
        return redirect(request.GET.get('next', reverse('spirit:profile-update')))

    if request.method == 'POST':
        form = ResendActivationForm(data=request.POST)

        if not request.is_limited and form.is_valid():
            user = form.get_user()
            send_activation_email(request, user)

        messages.info(request, _("If you don't receive an email, please make sure you've entered "
                                 "the address you registered with, and check your spam folder."))
        return redirect(reverse('spirit:user-login'))
    else:
        form = ResendActivationForm()

    return render(request, 'spirit/user/activation_resend.html', {'form': form, })


@login_required
def profile_update(request):
    if request.method == 'POST':
        form = UserProfileForm(data=request.POST, instance=request.user)

        if form.is_valid():
            form.save()
            messages.info(request, _("Your profile has been updated!"))
            return redirect(reverse('spirit:profile-update'))
    else:
        form = UserProfileForm(instance=request.user)

    return render(request, 'spirit/user/profile_update.html', {'form': form, })


@login_required
def profile_password_change(request):
    if request.method == 'POST':
        form = PasswordChangeForm(user=request.user, data=request.POST)

        if form.is_valid():
            form.save()
            messages.info(request, _("Your password has been changed!"))
            return redirect(reverse('spirit:profile-update'))
    else:
        form = PasswordChangeForm(user=request.user)

    return render(request, 'spirit/user/profile_password_change.html', {'form': form, })


@login_required
def profile_email_change(request):
    if request.method == 'POST':
        form = EmailChangeForm(user=request.user, data=request.POST)

        if form.is_valid():
            send_email_change_email(request, request.user, form.get_email())
            messages.info(request, _("We have sent you an email so you can confirm the change!"))
            return redirect(reverse('spirit:profile-update'))
    else:
        form = EmailChangeForm()

    return render(request, 'spirit/user/profile_email_change.html', {'form': form, })


@login_required
def email_change_confirm(request, token):
    user = request.user
    email_change = UserEmailChangeTokenGenerator()

    if email_change.is_valid(user, token):
        user.email = email_change.get_email()
        user.save()
        messages.info(request, _("Your email has been changed!"))

    return redirect(reverse('spirit:profile-update'))


@login_required
def profile_topics(request, pk, slug):
    p_user = get_object_or_404(User, pk=pk)

    if p_user.slug != slug:
        return HttpResponsePermanentRedirect(reverse("spirit:profile-topics", kwargs={'pk': p_user.pk,
                                                                                      'slug': p_user.slug}))

    topics = Topic.objects.for_public().filter(user=p_user).order_by('-date').select_related('user')

    return render(request, 'spirit/user/profile_topics.html', {'p_user': p_user, 'topics': topics})


@login_required
def profile_comments(request, pk, slug):
    p_user = get_object_or_404(User, pk=pk)

    if p_user.slug != slug:
        return HttpResponsePermanentRedirect(reverse("spirit:profile-detail", kwargs={'pk': p_user.pk,
                                                                                      'slug': p_user.slug}))

    comments = Comment.objects.for_user_public(user=p_user)
    return render(request, 'spirit/user/profile_comments.html', {'p_user': p_user, 'comments': comments})


@login_required
def profile_likes(request, pk, slug):
    p_user = get_object_or_404(User, pk=pk)

    if p_user.slug != slug:
        return HttpResponsePermanentRedirect(reverse("spirit:profile-likes", kwargs={'pk': p_user.pk,
                                                                                     'slug': p_user.slug}))

    comments = Comment.objects.for_public().filter(comment_likes__user=p_user).order_by('-comment_likes__date')
    return render(request, 'spirit/user/profile_likes.html', {'p_user': p_user, 'comments': comments})


@login_required
def user_menu(request):
    return render(request, 'spirit/user/menu.html')


#-*- coding: utf-8 -*-

import os
import json
from StringIO import StringIO

from django.test import TestCase, RequestFactory
from django.core.cache import cache
from django.core.urlresolvers import reverse
from django.template import Template, Context
from django.utils.translation import ugettext as _
from django.core.exceptions import PermissionDenied
from django.contrib.auth.models import User as UserModel
from django.contrib.auth import get_user_model
from django.conf import settings
from django.core.files.uploadedfile import SimpleUploadedFile
from django.test.utils import override_settings

import utils

from spirit.models.comment import Comment,\
    comment_like_post_create, comment_like_post_delete,\
    topic_post_moderate
from spirit.forms.comment import CommentForm, CommentMoveForm, CommentImageForm
from spirit.signals.comment import comment_post_update, comment_posted, comment_pre_update, comment_moved
from spirit.templatetags.tags.comment import render_comments_form
from spirit.utils import markdown
from spirit.views.comment import comment_delete
from spirit.models.topic import Topic
from spirit.models.category import Category


User = get_user_model()


class CommentViewTest(TestCase):

    fixtures = ['spirit_init.json', ]

    def setUp(self):
        cache.clear()
        self.user = utils.create_user()
        self.category = utils.create_category()
        self.topic = utils.create_topic(category=self.category, user=self.user)

    def test_comment_publish(self):
        """
        create comment
        """
        utils.login(self)
        form_data = {'comment': 'foobar', }
        response = self.client.post(reverse('spirit:comment-publish', kwargs={'topic_id': self.topic.pk, }),
                                    form_data)
        expected_url = reverse('spirit:comment-find', kwargs={'pk': 1, })
        self.assertRedirects(response, expected_url, status_code=302, target_status_code=302)
        self.assertEqual(len(Comment.objects.all()), 1)

        # ratelimit
        response = self.client.post(reverse('spirit:comment-publish', kwargs={'topic_id': self.topic.pk, }),
                                    form_data)
        self.assertEqual(len(Comment.objects.all()), 1)

        # get
        response = self.client.get(reverse('spirit:comment-publish', kwargs={'topic_id': self.topic.pk, }))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['topic'], self.topic)

    def test_comment_publish_on_private(self):
        """
        create comment on private topic
        """
        private = utils.create_private_topic(user=self.user)

        utils.login(self)
        form_data = {'comment': 'foobar', }
        response = self.client.post(reverse('spirit:comment-publish', kwargs={'topic_id': private.topic.pk, }),
                                    form_data)
        expected_url = reverse('spirit:comment-find', kwargs={'pk': 1, })
        self.assertRedirects(response, expected_url, status_code=302, target_status_code=302)
        self.assertEqual(len(Comment.objects.all()), 1)

    def test_comment_publish_on_closed_topic(self):
        """
        should not be able to create a comment on a closed topic
        """
        Topic.objects.filter(pk=self.topic.pk).update(is_closed=True)

        utils.login(self)
        form_data = {'comment': 'foobar', }
        response = self.client.post(reverse('spirit:comment-publish', kwargs={'topic_id': self.topic.pk, }),
                                    form_data)
        self.assertEqual(response.status_code, 404)

    def test_comment_publish_on_closed_cateory(self):
        """
        should be able to create a comment on a closed category (if topic is not closed)
        """
        Category.objects.filter(pk=self.category.pk).update(is_closed=True)

        utils.login(self)
        form_data = {'comment': 'foobar', }
        response = self.client.post(reverse('spirit:comment-publish', kwargs={'topic_id': self.topic.pk, }),
                                    form_data)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(len(Comment.objects.all()), 1)

    def test_comment_publish_on_removed_topic_or_category(self):
        """
        should not be able to create a comment
        """
        # removed category
        Category.objects.all().update(is_removed=True)

        utils.login(self)
        form_data = {'comment': 'foobar', }
        response = self.client.post(reverse('spirit:comment-publish', kwargs={'topic_id': self.topic.pk, }),
                                    form_data)
        self.assertEqual(response.status_code, 404)

        # removed subcategory
        Category.objects.all().update(is_removed=False)
        subcategory = utils.create_category(parent=self.category, is_removed=True)
        topic2 = utils.create_topic(subcategory)

        utils.login(self)
        form_data = {'comment': 'foobar', }
        response = self.client.post(reverse('spirit:comment-publish', kwargs={'topic_id': topic2.pk, }),
                                    form_data)
        self.assertEqual(response.status_code, 404)

        # removed topic
        Category.objects.all().update(is_removed=False)
        Topic.objects.all().update(is_removed=True)

        utils.login(self)
        form_data = {'comment': 'foobar', }
        response = self.client.post(reverse('spirit:comment-publish', kwargs={'topic_id': self.topic.pk, }),
                                    form_data)
        self.assertEqual(response.status_code, 404)

    def test_comment_publish_no_access(self):
        """
        should not be able to create a comment on a private topic if has no access
        """
        private = utils.create_private_topic(user=self.user)
        private.delete()

        utils.login(self)
        form_data = {'comment': 'foobar', }
        response = self.client.post(reverse('spirit:comment-publish', kwargs={'topic_id': private.topic.pk, }),
                                    form_data)
        self.assertEqual(response.status_code, 404)

    def test_comment_publish_signal(self):
        """
        create comment signal
        """
        def comment_posted_handler(sender, comment, **kwargs):
            self._comment = comment
        comment_posted.connect(comment_posted_handler)

        utils.login(self)
        form_data = {'comment': 'foobar', }
        response = self.client.post(reverse('spirit:comment-publish', kwargs={'topic_id': self.topic.pk, }),
                                    form_data)
        self.assertEqual(self._comment.comment, 'foobar')

    def test_comment_publish_quote(self):
        """
        create comment quote
        """
        utils.login(self)
        comment = utils.create_comment(topic=self.topic)
        response = self.client.get(reverse('spirit:comment-publish', kwargs={'topic_id': self.topic.pk,
                                                                             'pk': comment.pk}))
        self.assertEqual(response.context['form'].initial['comment'],
                         markdown.quotify(comment.comment, comment.user.username))

    def test_comment_publish_next(self):
        """
        next on create comment
        """
        utils.login(self)
        form_data = {'comment': 'foobar', 'next': '/fakepath/'}
        response = self.client.post(reverse('spirit:comment-publish', kwargs={'topic_id': self.topic.pk, }),
                                    form_data)
        self.assertRedirects(response, '/fakepath/', status_code=302, target_status_code=404)

    def test_comment_update(self):
        """
        update comment
        """
        comment = utils.create_comment(user=self.user, topic=self.topic)

        utils.login(self)
        form_data = {'comment': 'barfoo', }
        response = self.client.post(reverse('spirit:comment-update', kwargs={'pk': comment.pk, }),
                                    form_data)
        expected_url = reverse('spirit:comment-find', kwargs={'pk': 1, })
        self.assertRedirects(response, expected_url, status_code=302, target_status_code=302)
        self.assertEqual(Comment.objects.get(pk=comment.pk).comment, 'barfoo')

        # next
        form_data.update({'next': '/fakepath/', })
        response = self.client.post(reverse('spirit:comment-update', kwargs={'pk': comment.pk, }),
                                    form_data)
        self.assertRedirects(response, '/fakepath/', status_code=302, target_status_code=404)

    def test_comment_update_not_moderator(self):
        """
        non moderators can not update other people comments
        """
        user = utils.create_user()
        comment = utils.create_comment(user=user, topic=self.topic)

        utils.login(self)
        form_data = {'comment': 'barfoo', }
        response = self.client.post(reverse('spirit:comment-update', kwargs={'pk': comment.pk, }),
                                    form_data)
        self.assertEqual(response.status_code, 404)

    def test_comment_update_moderator(self):
        """
        moderators can update other people comments
        """
        User.objects.filter(pk=self.user.pk).update(is_moderator=True)
        user = utils.create_user()
        comment = utils.create_comment(user=user, topic=self.topic)

        utils.login(self)
        form_data = {'comment': 'barfoo', }
        response = self.client.post(reverse('spirit:comment-update', kwargs={'pk': comment.pk, }),
                                    form_data)
        expected_url = reverse('spirit:comment-find', kwargs={'pk': 1, })
        self.assertRedirects(response, expected_url, status_code=302, target_status_code=302)
        self.assertEqual(Comment.objects.get(pk=comment.pk).comment, 'barfoo')

    def test_comment_update_signal(self):
        """
        update comment, emit signal
        """
        def comment_pre_update_handler(sender, comment, **kwargs):
            self._comment_old = comment
        comment_pre_update.connect(comment_pre_update_handler)

        def comment_post_update_handler(sender, comment, **kwargs):
            self._comment_new = comment
        comment_post_update.connect(comment_post_update_handler)

        utils.login(self)
        comment_posted = utils.create_comment(user=self.user, topic=self.topic)
        form_data = {'comment': 'barfoo', }
        response = self.client.post(reverse('spirit:comment-update', kwargs={'pk': comment_posted.pk, }),
                                    form_data)
        self.assertEqual(repr(self._comment_new), repr(Comment.objects.get(pk=comment_posted.pk)))
        self.assertEqual(repr(self._comment_old), repr(comment_posted))

    def test_comment_delete_permission_denied_to_non_moderator(self):
        req = RequestFactory().get('/')
        req.user = UserModel()
        req.user.is_moderator = False
        self.assertRaises(PermissionDenied, comment_delete, req)

    def test_comment_delete(self):
        """
        comment delete
        """
        self.user = utils.create_user(is_moderator=True)
        comment = utils.create_comment(user=self.user, topic=self.topic)

        utils.login(self)
        form_data = {}
        response = self.client.post(reverse('spirit:comment-delete', kwargs={'pk': comment.pk, }),
                                    form_data)
        expected_url = comment.get_absolute_url()
        self.assertRedirects(response, expected_url, status_code=302, target_status_code=302)

        response = self.client.get(reverse('spirit:comment-delete', kwargs={'pk': comment.pk, }))
        self.assertEqual(response.status_code, 200)

    def test_comment_undelete(self):
        """
        comment undelete
        """
        self.user = utils.create_user(is_moderator=True)
        comment = utils.create_comment(user=self.user, topic=self.topic, is_removed=True)

        utils.login(self)
        form_data = {}
        response = self.client.post(reverse('spirit:comment-undelete', kwargs={'pk': comment.pk, }),
                                    form_data)
        expected_url = comment.get_absolute_url()
        self.assertRedirects(response, expected_url, status_code=302, target_status_code=302)

        response = self.client.get(reverse('spirit:comment-undelete', kwargs={'pk': comment.pk, }))
        self.assertEqual(response.status_code, 200)

    def test_comment_move(self):
        """
        comment move to another topic
        """
        utils.login(self)
        self.user.is_moderator = True
        self.user.save()
        comment = utils.create_comment(user=self.user, topic=self.topic)
        comment2 = utils.create_comment(user=self.user, topic=self.topic)
        to_topic = utils.create_topic(category=self.category)
        form_data = {'topic': to_topic.pk,
                     'comments': [comment.pk, comment2.pk], }
        response = self.client.post(reverse('spirit:comment-move', kwargs={'topic_id': self.topic.pk, }),
                                    form_data)
        expected_url = self.topic.get_absolute_url()
        self.assertRedirects(response, expected_url, status_code=302)
        self.assertEqual(Comment.objects.filter(topic=to_topic.pk).count(), 2)
        self.assertEqual(Comment.objects.filter(topic=self.topic.pk).count(), 0)

    def test_comment_move_signal(self):
        """
        move comments, emit signal
        """
        self._comments = []

        def comment_posted_handler(sender, comment, **kwargs):
            self._comments.append(comment)
        comment_posted.connect(comment_posted_handler)

        def comment_moved_handler(sender, comments, topic_from, **kwargs):
            self._comment_count = len(comments)
            self._topic_from = topic_from
        comment_moved.connect(comment_moved_handler)

        utils.login(self)
        self.user.is_moderator = True
        self.user.save()

        comment = utils.create_comment(user=self.user, topic=self.topic)
        comment2 = utils.create_comment(user=self.user, topic=self.topic)
        to_topic = utils.create_topic(self.category)

        form_data = {'topic': to_topic.pk,
                     'comments': [comment.pk, comment2.pk], }
        response = self.client.post(reverse('spirit:comment-move', kwargs={'topic_id': self.topic.pk, }),
                                    form_data)
        self.assertEqual(response.status_code, 302)
        self.assertListEqual(self._comments, [comment2, comment])
        self.assertEqual(self._comment_count, 2)
        self.assertEqual(repr(self._topic_from), repr(self.topic))

    def test_comment_find(self):
        """
        comment absolute and lazy url
        """
        comment = utils.create_comment(user=self.user, topic=self.topic)
        response = self.client.post(reverse('spirit:comment-find', kwargs={'pk': comment.pk, }))
        expected_url = comment.topic.get_absolute_url() + "#c%d" % comment.pk
        self.assertRedirects(response, expected_url, status_code=302)

    def test_comment_image_upload(self):
        """
        comment image upload
        """
        utils.login(self)
        img = StringIO('GIF87a\x01\x00\x01\x00\x80\x01\x00\x00\x00\x00ccc,\x00'
                       '\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;')
        files = {'image': SimpleUploadedFile('image.gif', img.read(), content_type='image/gif'), }
        response = self.client.post(reverse('spirit:comment-image-upload-ajax'),
                                    HTTP_X_REQUESTED_WITH='XMLHttpRequest',
                                    data=files)
        res = json.loads(response.content)
        self.assertEqual(res['url'], os.path.join(settings.MEDIA_URL, 'spirit', 'images', str(self.user.pk),
                                                  "bf21c3043d749d5598366c26e7e4ab44.gif").replace("\\", "/"))
        os.remove(os.path.join(settings.MEDIA_ROOT, 'spirit', 'images', str(self.user.pk),
                               "bf21c3043d749d5598366c26e7e4ab44.gif"))

    def test_comment_image_upload_invalid(self):
        """
        comment image upload, invalid image
        """
        utils.login(self)
        image = StringIO('BAD\x02D\x01\x00;')
        image.name = 'image.gif'
        image.content_type = 'image/gif'
        files = {'image': SimpleUploadedFile(image.name, image.read()), }
        response = self.client.post(reverse('spirit:comment-image-upload-ajax'),
                                    HTTP_X_REQUESTED_WITH='XMLHttpRequest',
                                    data=files)
        res = json.loads(response.content)
        self.assertIn('error', res.keys())
        self.assertIn('image', res['error'].keys())


class CommentSignalTest(TestCase):

    fixtures = ['spirit_init.json', ]

    def setUp(self):
        cache.clear()
        self.user = utils.create_user()
        self.category = utils.create_category()
        self.topic = utils.create_topic(category=self.category, user=self.user)

    def test_comment_comment_post_update_handler(self):
        """
        Increase modified_count on updated comment
        """
        comment = utils.create_comment(topic=self.topic)
        comment_post_update.send(sender=comment.__class__, comment=comment)
        self.assertEqual(Comment.objects.get(pk=comment.pk).modified_count, 1)

    def test_comment_comment_like_post_create_handler(self):
        """
        Increase like_count on comment like
        """
        comment = utils.create_comment(topic=self.topic)
        comment_like_post_create.send(sender=comment.__class__, comment=comment)
        self.assertEqual(Comment.objects.get(pk=comment.pk).likes_count, 1)

    def test_comment_comment_like_post_delete_handler(self):
        """
        Decrease like_count on remove comment like
        """
        comment = utils.create_comment(topic=self.topic)
        comment_like_post_create.send(sender=comment.__class__, comment=comment)
        self.assertEqual(Comment.objects.get(pk=comment.pk).likes_count, 1)
        comment_like_post_delete.send(sender=comment.__class__, comment=comment)
        self.assertEqual(Comment.objects.get(pk=comment.pk).likes_count, 0)

    def test_topic_post_moderate_handler(self):
        """
        Create comment that tells what moderation action was made
        """
        topic_post_moderate.send(sender=None, user=self.user, topic=self.topic, action=1)
        self.assertEqual(Comment.objects.filter(user=self.user, topic=self.topic, action=1).count(), 1)


class CommentTemplateTagTests(TestCase):

    fixtures = ['spirit_init.json', ]

    def setUp(self):
        cache.clear()
        self.user = utils.create_user()
        self.category = utils.create_category()
        self.topic = utils.create_topic(category=self.category, user=self.user)
        utils.create_comment(topic=self.topic)
        utils.create_comment(topic=self.topic)
        utils.create_comment(topic=self.topic)

    def test_get_comment_list(self):
        """
        should display all comment for a topic
        """
        out = Template(
            "{% load spirit_tags %}"
            "{% get_comment_list topic as comments %}"
            "{% for c in comments %}"
            "{{ c.comment }},"
            "{% endfor %}"
        ).render(Context({'topic': self.topic, }))
        self.assertEqual(out, "foobar0,foobar1,foobar2,")

    def test_render_comments_form(self):
        """
        should display simple comment form
        """
        out = Template(
            "{% load spirit_tags %}"
            "{% render_comments_form topic %}"
        ).render(Context({'topic': self.topic, }))
        context = render_comments_form(self.topic)
        self.assertEqual(context['next'], None)
        self.assertIsInstance(context['form'], CommentForm)
        self.assertEqual(context['topic_id'], self.topic.pk)

    def test_get_action_text(self):
        """
        should display action
        """
        out = Template(
            "{% load spirit_tags %}"
            "{% get_comment_action_text 1 %}"
        ).render(Context())
        self.assertNotEqual(out, "")


class CommentFormTest(TestCase):

    fixtures = ['spirit_init.json', ]

    def setUp(self):
        cache.clear()
        self.user = utils.create_user()
        self.category = utils.create_category()
        self.topic = utils.create_topic(category=self.category)

    def test_comment_create(self):
        form_data = {'comment': 'foo', }
        form = CommentForm(data=form_data)
        self.assertEqual(form.is_valid(), True)

    def test_comment_markdown(self):
        form_data = {'comment': u'**Spirit unicode: áéíóú** '
                                u'<script>alert();</script>', }
        form = CommentForm(data=form_data)
        self.assertEqual(form.is_valid(), True)
        form.user = self.user
        form.topic = self.topic
        comment = form.save()
        self.assertEqual(comment.comment_html, u'<p><strong>Spirit unicode: áéíóú</strong> '
                                               u'&lt;script&gt;alert();&lt;/script&gt;</p>')

    def test_comments_move(self):
        comment = utils.create_comment(user=self.user, topic=self.topic)
        comment2 = utils.create_comment(user=self.user, topic=self.topic)
        to_topic = utils.create_topic(category=self.category)
        form_data = {'topic': to_topic.pk,
                     'comments': [comment.pk, comment2.pk], }
        form = CommentMoveForm(topic=self.topic, data=form_data)
        self.assertEqual(form.is_valid(), True)
        self.assertEqual(form.save(), list(Comment.objects.filter(topic=to_topic)))

    def test_comment_image_upload(self):
        """
        Image upload
        """
        content = 'GIF87a\x01\x00\x01\x00\x80\x01\x00\x00\x00\x00ccc,\x00' \
                  '\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;'
        img = StringIO(content)
        files = {'image': SimpleUploadedFile('image.gif', img.read(), content_type='image/gif'), }

        form = CommentImageForm(user=self.user, data={}, files=files)
        self.assertTrue(form.is_valid())
        image = form.save()
        self.assertEqual(image.name, "bf21c3043d749d5598366c26e7e4ab44.gif")
        image_url = os.path.join(settings.MEDIA_URL, 'spirit', 'images', str(self.user.pk),
                                 image.name).replace("\\", "/")
        self.assertEqual(image.url, image_url)
        image_path = os.path.join(settings.MEDIA_ROOT, 'spirit', 'images', str(self.user.pk), image.name)
        self.assertTrue(os.path.isfile(image_path))
        image.open()
        self.assertEqual(image.read(), content)

        with open(image_path, "rb") as fh:
            self.assertEqual(fh.read(), content)

        os.remove(image_path)

    def test_comment_image_upload_no_extension(self):
        """
        Image upload no extension
        """
        img = StringIO('GIF87a\x01\x00\x01\x00\x80\x01\x00\x00\x00\x00ccc,\x00'
                       '\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;')
        files = {'image': SimpleUploadedFile('image', img.read(), content_type='image/gif'), }
        form = CommentImageForm(user=self.user, data={}, files=files)
        self.assertTrue(form.is_valid())
        image = form.save()
        self.assertEqual(image.name, "bf21c3043d749d5598366c26e7e4ab44.gif")
        os.remove(os.path.join(settings.MEDIA_ROOT, 'spirit', 'images', str(self.user.pk), image.name))

    @override_settings(ST_ALLOWED_UPLOAD_IMAGE_FORMAT=['png', ])
    def test_comment_image_upload_not_allowed_format(self):
        """
        Image upload, invalid format
        """
        img = StringIO('GIF87a\x01\x00\x01\x00\x80\x01\x00\x00\x00\x00ccc,\x00'
                       '\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;')
        # fake png extension
        files = {'image': SimpleUploadedFile('image.png', img.read(), content_type='image/png'), }
        form = CommentImageForm(data={}, files=files)
        self.assertFalse(form.is_valid())

    def test_comment_image_upload_invalid(self):
        """
        Image upload, bad image
        """
        img = StringIO('bad\x00;')
        files = {'image': SimpleUploadedFile('image.gif', img.read(), content_type='image/gif'), }
        form = CommentImageForm(data={}, files=files)
        self.assertFalse(form.is_valid())


#-*- coding: utf-8 -*-

from . import register
from spirit.models.topic_favorite import TopicFavorite
from spirit.forms.topic_favorite import FavoriteForm


@register.inclusion_tag('spirit/topic_favorite/_form.html')
def render_favorite_form(topic, user, next=None):
    try:
        favorite = TopicFavorite.objects.get(user=user, topic=topic)
    except TopicFavorite.DoesNotExist:
        favorite = None

    form = FavoriteForm()
    return {'form': form, 'topic_id': topic.pk, 'favorite': favorite, 'next': next}


#-*- coding: utf-8 -*-

import re

from markdown.extensions import Extension
from markdown.preprocessors import Preprocessor


# Try to get the video ID. Works for URLs of the form:
# * https://www.youtube.com/watch?v=Z0UISCEe52Y
# * http://youtu.be/afyK1HSFfgw
# * https://www.youtube.com/embed/vsF0K3Ou1v0
PATTERN_RE = ur'^https?://(www\.)?(youtube\.com/watch\?v=|youtu\.be/|youtube\.com/embed/)(?P<id>[a-zA-Z0-9_\-]{11})$'


class YouTubefyExtension(Extension):

    def extendMarkdown(self, md, md_globals):
        md.registerExtension(self)
        md.preprocessors.add('youtubefy',
                             YouTubefyPreprocessor(md),
                             '_end')


class YouTubefyPreprocessor(Preprocessor):

    def run(self, lines):
        new_lines = []

        def youtubefy(match):
            video_id = match.group("id")
            html = u'<span class="video"><iframe src="https://www.youtube.com/embed/{video_id}?feature=oembed" ' \
                   u'allowfullscreen></iframe></span>'.format(video_id=video_id)
            return self.markdown.htmlStash.store(html, safe=True)

        for line in lines:
            if line.strip():
                line = re.sub(PATTERN_RE, youtubefy, line, flags=re.UNICODE)

            new_lines.append(line)

        return new_lines


def makeExtension(configs=None):
    return YouTubefyExtension(configs=configs)


#-*- coding: utf-8 -*-

from django.contrib.auth.decorators import login_required
from django.shortcuts import render, get_object_or_404

from ..models.comment_history import CommentHistory
from ..models.comment import Comment


@login_required
def comment_history_detail(request, comment_id):
    comment = get_object_or_404(Comment.objects.for_access(request.user), pk=comment_id)
    comments = CommentHistory.objects.filter(comment_fk=comment)\
        .order_by('date')
    return render(request, 'spirit/comment_history/detail.html', {'comments': comments, })



#-*- coding: utf-8 -*-

from django.conf.urls import patterns, url


urlpatterns = patterns("spirit.views.admin.comment_flag",
    url(r'^$', 'flag_open', name='admin-flag'),
    url(r'^open/$', 'flag_open', name='admin-flag-open'),
    url(r'^closed/$', 'flag_closed', name='admin-flag-closed'),
    url(r'^(?P<pk>\d+)/$', 'flag_detail', name='admin-flag-detail'),
)


#-*- coding: utf-8 -*-

import pytz

from django.utils import timezone
from django.contrib.auth import get_user_model
from django.conf import settings
from django.contrib.auth import logout
from django.core.urlresolvers import resolve
from django.contrib.auth.views import redirect_to_login


User = get_user_model()


class XForwardedForMiddleware(object):

    def process_request(self, request):
        if not settings.DEBUG:
            request.META['REMOTE_ADDR'] = request.META['HTTP_X_FORWARDED_FOR'].split(",")[-1].strip()


class TimezoneMiddleware(object):

    def process_request(self, request):
        if request.user.is_authenticated():
            timezone.activate(request.user.timezone)
        else:
            timezone.deactivate()


class LastIPMiddleware(object):

    def process_request(self, request):
        if not request.user.is_authenticated():
            return

        last_ip = request.META['REMOTE_ADDR'].strip()

        if request.user.last_ip == last_ip:
            return

        User.objects.filter(pk=request.user.pk)\
            .update(last_ip=last_ip)


class LastSeenMiddleware(object):

    def process_request(self, request):
        if not request.user.is_authenticated():
            return

        threshold = settings.ST_USER_LAST_SEEN_THRESHOLD_MINUTES * 60
        delta = timezone.now() - request.user.last_seen

        if delta.seconds < threshold:
            return

        User.objects.filter(pk=request.user.pk)\
            .update(last_seen=timezone.now())


class ActiveUserMiddleware(object):

    def process_request(self, request):
        if not request.user.is_authenticated():
            return

        if not request.user.is_active:
           logout(request)


class PrivateForumMiddleware(object):

    def process_request(self, request):
        if not settings.ST_PRIVATE_FORUM:
            return

        if request.user.is_authenticated():
            return

        resolver_match = resolve(request.path)

        if resolver_match.app_name != 'spirit':
            return

        # Namespacing /user/ would be better but breaks current urls namespace.
        url_whitelist = ['user-login',
                         'user-logout',
                         'user-register',
                         'resend-activation',
                         'registration-activation',
                         'password-reset',
                         'password-reset-done',
                         'password-reset-confirm',
                         'password-reset-complete']

        if resolver_match.url_name in url_whitelist:
            return

        return redirect_to_login(next=request.get_full_path(),
                                 login_url=settings.LOGIN_URL)


#-*- coding: utf-8 -*-

from . import register
from spirit.models.comment_bookmark import CommentBookmark


@register.simple_tag()
def populate_bookmarks(topics, user, to_attr='bookmark'):
    bookmarks = CommentBookmark.objects.filter(topic__in=topics, user=user)\
        .select_related('topic')
    bookmarks_dict = {b.topic_id: b for b in bookmarks}

    for t in topics:
        setattr(t, to_attr, bookmarks_dict.get(t.pk))

    return ""


#-*- coding: utf-8 -*-

from django.conf.urls import patterns, url


urlpatterns = patterns("spirit.views.topic_notification",
    url(r'^$', 'notification_list_unread', name='topic-notification-list-unread'),
    url(r'^list/$', 'notification_list', name='topic-notification-list'),
    url(r'^(?P<topic_id>\d+)/create/$', 'notification_create', name='topic-notification-create'),
    url(r'^(?P<pk>\d+)/update/$', 'notification_update', name='topic-notification-update'),
    url(r'^ajax/$', 'notification_ajax', name='topic-notification-ajax'),
)


#-*- coding: utf-8 -*-

from django.contrib.auth import get_user_model
from django.conf import settings

from spirit.models.topic import Topic
from spirit.models.category import Category
from spirit.models.comment import Comment
from spirit.models.topic_private import TopicPrivate

User = get_user_model()


def create_user(**kwargs):
    if 'username' not in kwargs:
        kwargs['username'] = "foo%d" % User.objects.all().count()

    if 'email' not in kwargs:
        kwargs['email'] = "%s@bar.com" % kwargs['username']

    if 'password' not in kwargs:
        kwargs['password'] = "bar"

    return User.objects.create_user(**kwargs)


def create_topic(category, **kwargs):
    if 'user' not in kwargs:
        username = "foo%d" % User.objects.all().count()
        email = "%s@bar.com" % username
        kwargs['user'] = User.objects.create_user(username=username, password="bar", email=email)

    if 'title' not in kwargs:
        kwargs['title'] = "foo%d" % Topic.objects.all().count()

    return Topic.objects.create(category=category, **kwargs)


def create_private_topic(**kwargs):
    assert 'category' not in kwargs, "do not pass category param"

    category = Category.objects.get(pk=settings.ST_TOPIC_PRIVATE_CATEGORY_PK)
    topic = create_topic(category=category, **kwargs)
    return TopicPrivate.objects.create(topic=topic, user=topic.user)


def create_category(**kwargs):
    if 'title' not in kwargs:
        kwargs['title'] = "foo%d" % Category.objects.all().count()

    return Category.objects.create(**kwargs)


def create_subcategory(category, **kwargs):
    if 'title' not in kwargs:
        kwargs['title'] = "foo%d" % Category.objects.all().count()

    return Category.objects.create(parent=category, **kwargs)


def create_comment(**kwargs):
    if 'comment' not in kwargs:
        kwargs['comment'] = "foobar%d" % Comment.objects.all().count()

    if 'comment_html' not in kwargs:
        kwargs['comment_html'] = kwargs['comment']

    if 'user' not in kwargs:
        kwargs['user'] = create_user()

    return Comment.objects.create(**kwargs)


def login(test_case_instance, user=None, password=None):
    user = user or test_case_instance.user
    password = password or "bar"
    login_successful = test_case_instance.client.login(username=user.username, password=password)
    test_case_instance.assertTrue(login_successful)


#-*- coding: utf-8 -*-

from django.shortcuts import render

from spirit.utils.decorators import administrator_required
from spirit.models.topic import Topic


@administrator_required
def topic_deleted(request):
    # Private topics cant be deleted, closed or pinned so we are ok
    topics = Topic.objects.filter(is_removed=True)
    return render(request, 'spirit/admin/topic/topic_deleted.html', {'topics': topics, })


@administrator_required
def topic_closed(request):
    topics = Topic.objects.filter(is_closed=True)
    return render(request, 'spirit/admin/topic/topic_closed.html', {'topics': topics, })


@administrator_required
def topic_pinned(request):
    topics = Topic.objects.filter(is_pinned=True)
    return render(request, 'spirit/admin/topic/topic_pinned.html', {'topics': topics, })


#-*- coding: utf-8 -*-


def quotify(comment, username):
    """
    Converts 'Foo\nbar' to:
    @username
    > Foo
    > bar
    \n\n
    """
    header = u"@%s" % username
    lines = comment.splitlines()
    quote = u"\n> ".join(lines)
    quote = u"%(header)s\n> %(quote)s\n\n" % ({'header': header, 'quote': quote})
    return quote


#-*- coding: utf-8 -*-

from django.conf.urls import patterns, include, url

import djconfig

from spirit.forms.admin import BasicConfigForm

# TODO: use app loader in django 1.7
djconfig.register(BasicConfigForm)

urlpatterns = patterns('',
    url(r'^$', 'spirit.views.topic.topics_active', name='index'),
    url(r'^st/admin/', include('spirit.urls.admin')),
    url(r'^category/', include('spirit.urls.category')),
    url(r'^topic/', include('spirit.urls.topic')),
    url(r'^topic/unread/', include('spirit.urls.topic_unread')),
    url(r'^topic/notification/', include('spirit.urls.topic_notification')),
    url(r'^topic/favorite/', include('spirit.urls.topic_favorite')),
    url(r'^topic/private/', include('spirit.urls.topic_private')),
    url(r'^topic/poll/', include('spirit.urls.topic_poll')),
    url(r'^comment/', include('spirit.urls.comment')),
    url(r'^comment/bookmark/', include('spirit.urls.comment_bookmark')),
    url(r'^comment/flag/', include('spirit.urls.comment_flag')),
    url(r'^comment/history/', include('spirit.urls.comment_history')),
    url(r'^comment/like/', include('spirit.urls.comment_like')),
    url(r'^user/', include('spirit.urls.user')),
    url(r'^search/', include('spirit.urls.search')),
)


#-*- coding: utf-8 -*-

from . import register

from spirit.models.topic_notification import TopicNotification
from spirit.forms.topic_notification import NotificationForm


@register.assignment_tag()
def has_topic_notifications(user):
    return TopicNotification.objects.for_access(user=user)\
        .filter(is_read=False)\
        .exists()


@register.inclusion_tag('spirit/topic_notification/_form.html')
def render_notification_form(user, topic, next=None):
    try:
        notification = TopicNotification.objects.get(user=user, topic=topic)
    except TopicNotification.DoesNotExist:
        notification = None

    initial = {}

    if notification:
        initial['is_active'] = not notification.is_active

    form = NotificationForm(initial=initial)
    return {'form': form, 'topic_id': topic.pk, 'notification': notification, 'next': next}


#-*- coding: utf-8 -*-

from django.db.models import Manager
from django.db.models import Q


class TopicNotificationManager(Manager):

    def _for_all(self):
        return self.filter(Q(topic__category__parent=None) | Q(topic__category__parent__is_removed=False),
                           topic__category__is_removed=False,
                           topic__is_removed=False)

    def for_access(self, user):
        return self._for_all()\
            .filter(Q(topic__category__is_private=False) | Q(topic__topics_private__user=user),
                    user=user)\
            .exclude(comment=None)

    def read(self, user):
        # returns updated rows count (int)
        return self.filter(user=user)\
            .update(is_read=True)


#-*- coding: utf-8 -*-

import re

from markdown.extensions import Extension
from markdown.preprocessors import Preprocessor

from django.utils.html import escape as html_escape


PATTERN_RE = ur'^https?://[^\s]+\.(mp3|ogg|wav)(\?[^\s]+)?$'


class AudiofyExtension(Extension):

    def extendMarkdown(self, md, md_globals):
        md.registerExtension(self)
        md.preprocessors.add('audiofy',
                             AudiofyPreprocessor(md),
                             '_end')


class AudiofyPreprocessor(Preprocessor):

    def run(self, lines):
        new_lines = []

        def audiofy(match):
            url = match.group(0)
            url = html_escape(url)
            html = u'<audio controls><source src="{url}"><a href="{url}">{url}</a></audio>'.format(url=url)
            return self.markdown.htmlStash.store(html, safe=True)

        for line in lines:
            if line.strip():
                line = re.sub(PATTERN_RE, audiofy, line, flags=re.UNICODE)

            new_lines.append(line)

        return new_lines


def makeExtension(configs=None):
    return AudiofyExtension(configs=configs)


#-*- coding: utf-8 -*-

from django.db.models import Manager
from django.shortcuts import get_object_or_404
from django.db.models import Q


class TopicManager(Manager):

    def _for_all(self):
        return self.filter(Q(category__parent=None) | Q(category__parent__is_removed=False),
                           category__is_removed=False,
                           is_removed=False)

    def for_public(self):
        return self._for_all()\
            .filter(category__is_private=False)

    def for_public_open(self):
        return self.for_public()\
            .filter(is_closed=False)

    def for_category(self, category):
        if category.is_subcategory:
            return self.filter(category=category,
                               is_removed=False)
        else:
            return self.filter(Q(category=category) | Q(category__parent=category),
                               category__is_removed=False,
                               is_removed=False)

    def get_public_or_404(self, pk, user):
        if user.is_authenticated() and user.is_moderator:
            return get_object_or_404(self
                                     .select_related('category__parent'),
                                     pk=pk,
                                     category__is_private=False)
        else:
            return get_object_or_404(self.for_public()
                                     .select_related('category__parent'),
                                     pk=pk)

    def for_update_or_404(self, pk, user):
        if user.is_moderator:
            return get_object_or_404(self,
                                     pk=pk,
                                     category__is_private=False)
        else:
            return get_object_or_404(self.for_public_open(),
                                     pk=pk,
                                     user=user)

    def for_access(self, user):
        return self._for_all()\
            .filter(Q(category__is_private=False) | Q(topics_private__user=user))

    def for_access_open(self, user):
        return self.for_access(user)\
            .filter(is_closed=False)


#-*- coding: utf-8 -*-

from django.conf.urls import patterns, url


urlpatterns = patterns("spirit.views.comment_flag",
    url(r'^(?P<comment_id>\d+)/create/$', 'flag_create', name='flag-create'),
)


#-*- coding: utf-8 -*-

from django.conf.urls import patterns, url


urlpatterns = patterns("spirit.views.admin.user",
    url(r'^$', 'user_list', name='admin-user'),
    url(r'^edit/(?P<user_id>\d+)/$', 'user_edit', name='admin-user-edit'),
    url(r'^list/$', 'user_list', name='admin-user-list'),
    url(r'^admins/$', 'user_admins', name='admin-user-admins'),
    url(r'^mods/$', 'user_mods', name='admin-user-mods'),
    url(r'^unactive/$', 'user_unactive', name='admin-user-unactive'),
)


#-*- coding: utf-8 -*-

from django.http import Http404
from django.core.paginator import Paginator, InvalidPage

from .. import register
from spirit.utils.paginator.yt_paginator import YTPaginator


def _get_page(context, object_list, per_page, page_var, page_number, paginator_class):
    request = context["request"]
    page_number = page_number or request.GET.get(page_var, 1)

    paginator = paginator_class(object_list, per_page)

    try:
        page = paginator.page(page_number)
    except InvalidPage as err:
        raise Http404(err)

    return page


def _render_paginator(context, page, page_var, hashtag):
    query_dict = context["request"].GET.copy()

    try:
        del query_dict[page_var]
    except KeyError:
        pass

    extra_query = ""

    if query_dict:
        extra_query = "&%s" % query_dict.urlencode()

    if hashtag:
        hashtag = "#%s" % hashtag

    return {
        "page": page,
        "page_var": page_var,
        "hashtag": hashtag,
        "extra_query": extra_query
    }


@register.assignment_tag(takes_context=True)
def yt_paginator_autopaginate(context, object_list, per_page=15, page_var='page', page_number=None):
    return _get_page(context, object_list, per_page, page_var, page_number, YTPaginator)


@register.inclusion_tag("spirit/paginator/_yt_paginator.html", takes_context=True)
def render_yt_paginator(context, page, page_var='page', hashtag=''):
    return _render_paginator(context, page, page_var, hashtag)


@register.assignment_tag(takes_context=True)
def paginator_autopaginate(context, object_list, per_page=15, page_var='page', page_number=None):
    return _get_page(context, object_list, per_page, page_var, page_number, Paginator)


@register.inclusion_tag("spirit/paginator/_paginator.html", takes_context=True)
def render_paginator(context, page, page_var='page', hashtag=''):
    return _render_paginator(context, page, page_var, hashtag)


#-*- coding: utf-8 -*-

from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from haystack.forms import SearchForm
from haystack.query import EmptySearchQuerySet

from spirit.models.topic import Topic
from spirit.models.category import Category


class BaseSearchForm(SearchForm):

    def clean_q(self):
        q = self.cleaned_data['q']

        if len(q) < settings.ST_SEARCH_QUERY_MIN_LEN:
            raise forms.ValidationError(_("Your search must contain at least %(length)s characters.")
                                        % {'length': settings.ST_SEARCH_QUERY_MIN_LEN, })

        return q


class BasicSearchForm(BaseSearchForm):

    def search(self):
        sqs = super(BasicSearchForm, self).search()

        if isinstance(sqs, EmptySearchQuerySet):
            return sqs

        topics = sqs.models(Topic)
        return topics.filter(is_removed=False, is_category_removed=False, is_subcategory_removed=False)


class AdvancedSearchForm(BaseSearchForm):

    category = forms.ModelMultipleChoiceField(queryset=Category.objects.for_public(),
                                              required=False,
                                              label=_('Filter by'),
                                              widget=forms.CheckboxSelectMultiple)

    def search(self):
        sqs = super(AdvancedSearchForm, self).search()

        if isinstance(sqs, EmptySearchQuerySet):
            return sqs

        topics = sqs.models(Topic)
        categories = self.cleaned_data['category']

        if categories:
            topics = topics.filter(category_id__in=[c.pk for c in categories])

        return topics.filter(is_removed=False, is_category_removed=False, is_subcategory_removed=False)


#-*- coding: utf-8 -*-

from django import forms
from django.forms.models import inlineformset_factory, BaseInlineFormSet
from django.utils.translation import ugettext_lazy as _

from spirit.models.topic_poll import TopicPollChoice, TopicPoll, TopicPollVote


class TopicPollForm(forms.ModelForm):

    class Meta:
        model = TopicPoll
        fields = ['choice_limit', ]

    def __init__(self, topic=None, *args, **kwargs):
        super(TopicPollForm, self).__init__(*args, **kwargs)
        self.topic = topic

    def clean_choice_limit(self):
        choice_limit = self.cleaned_data['choice_limit']

        if choice_limit < 1:
            raise forms.ValidationError(_("This must be greater than zero"))

        return choice_limit

    def save(self, commit=True):
        if not self.instance.pk:
            self.instance.topic = self.topic

        return super(TopicPollForm, self).save(commit)


class TopicPollChoiceInlineFormSet(BaseInlineFormSet):

    def is_filled(self):
        for form in self.forms:
            description = form.cleaned_data.get('description')
            is_marked_as_delete = form.cleaned_data.get('DELETE', False)

            if description and not is_marked_as_delete:
                return True

        return False


# TODO: use min_num and validate_min in Django 1.7
TopicPollChoiceFormSet = inlineformset_factory(TopicPoll, TopicPollChoice,
                                               formset=TopicPollChoiceInlineFormSet, fields=('description', ),
                                               extra=2, max_num=20, validate_max=True)


class TopicPollVoteManyForm(forms.Form):
    """
    This special form allows single vote and multi vote as well.
    Its beauty is that it doesn't care if the choice_limit is increased or decreased later.
    """
    def __init__(self, user=None, poll=None, *args, **kwargs):
        super(TopicPollVoteManyForm, self).__init__(*args, **kwargs)
        self.user = user
        self.poll = poll
        choices = TopicPollChoice.objects.filter(poll=poll)

        if poll.choice_limit > 1:
            self.fields['choices'] = forms.ModelMultipleChoiceField(queryset=choices,
                                                                    widget=forms.CheckboxSelectMultiple,
                                                                    label=_("Poll choices"))
        else:
            self.fields['choices'] = forms.ModelChoiceField(queryset=choices,
                                                            empty_label=None,
                                                            widget=forms.RadioSelect,
                                                            label=_("Poll choices"))

    def load_initial(self):
        selected_choices = TopicPollChoice.objects.filter(poll=self.poll, votes__user=self.user)

        if self.poll.choice_limit == 1:
            try:
                selected_choices = selected_choices[0]
            except IndexError:
                selected_choices = None

        self.initial = {'choices': selected_choices, }

    def clean_choices(self):
        choices = self.cleaned_data['choices']

        if self.poll.choice_limit > 1:
            if len(choices) > self.poll.choice_limit:
                raise forms.ValidationError(_("Too many selected choices. Limit is %s")
                                            % self.poll.choice_limit)

        return choices

    def clean(self):
        cleaned_data = super(TopicPollVoteManyForm, self).clean()

        if self.poll.is_closed:
            raise forms.ValidationError(_("This poll is closed"))

        return cleaned_data

    def save_m2m(self):
        choices = self.cleaned_data['choices']

        if self.poll.choice_limit == 1:
            choices = [choices, ]

        TopicPollVote.objects.filter(user=self.user, choice__poll=self.poll)\
            .delete()

        return TopicPollVote.objects.bulk_create([TopicPollVote(user=self.user, choice=choice)
                                                 for choice in choices])


#-*- coding: utf-8 -*-

from django.conf import settings
from django.utils.translation import ugettext as _

from . import register
from spirit.forms.comment import CommentForm
from spirit.models.comment import Comment, MOVED, CLOSED, UNCLOSED, PINNED, UNPINNED


@register.assignment_tag()
def get_comment_list(topic):
    return Comment.objects.for_topic(topic).order_by('date')


@register.inclusion_tag('spirit/comment/_form.html')
def render_comments_form(topic, next=None):
    form = CommentForm()
    return {'form': form, 'topic_id': topic.pk, 'next': next, 'STATIC_URL': settings.STATIC_URL}


@register.simple_tag()
def get_comment_action_text(action):
    if action == MOVED:
        return _("This topic has been moved")
    elif action == CLOSED:
        return _("This topic has been closed")
    elif action == UNCLOSED:
        return _("This topic has been unclosed")
    elif action == PINNED:
        return _("This topic has been pinned")
    elif action == UNPINNED:
        return _("This topic has been unpinned")
    else:
        return _("Unknown topic moderation action")


#-*- coding: utf-8 -*-

# MINIMAL CONFIGURATION FOR PRODUCTION ENV

DEBUG = False

TEMPLATE_DEBUG = False

# https://docs.djangoproject.com/en/dev/ref/settings/#admins
ADMINS = (('John', 'john@example.com'), )

# Secret key generator: https://djskgen.herokuapp.com/
SECRET_KEY = ''

# https://docs.djangoproject.com/en/dev/ref/settings/#allowed-hosts
ALLOWED_HOSTS = ['.example.com', ]

# https://docs.djangoproject.com/en/dev/ref/settings/#databases
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'mydatabase',
        'USER': 'mydatabaseuser',
        'PASSWORD': 'mypassword',
        'HOST': '127.0.0.1',
        'PORT': '5432',
    }
}

# These are all the languages Spirit provides.
# https://www.transifex.com/projects/p/spirit/
gettext_noop = lambda s: s
LANGUAGES = (
    ('de', gettext_noop('German')),
    ('en', gettext_noop('English')),
    ('es', gettext_noop('Spanish')),
    ('sv', gettext_noop('Swedish')),
)

# Default language
LANGUAGE_CODE = 'en'


from django.conf.urls import patterns, include, url
from django.conf import settings
from django.conf.urls.static import static

# Uncomment the next two lines to enable the django admin:
from django.contrib import admin
admin.autodiscover()

# Override admin login for security purposes
from django.contrib.auth.decorators import login_required
admin.site.login = login_required(admin.site.login)


urlpatterns = patterns('',
    # Examples:
    # url(r'^$', 'example.views.home', name='home'),
    # url(r'^example/', include('example.foo.urls')),

    url(r'^', include('spirit.urls', namespace="spirit", app_name="spirit")),

    # Uncomment the admin/doc line below to enable admin documentation:
    # url(r'^admin/doc/', include('django.contrib.admindocs.urls')),

    # Uncomment the next line to enable the admin:
    url(r'^admin/', include(admin.site.urls)),
)

if settings.DEBUG:
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)


#-*- coding: utf-8 -*-

import re

from markdown.extensions import Extension
from markdown.preprocessors import Preprocessor

from django.contrib.auth import get_user_model
from django.conf import settings


User = get_user_model()


class MentionifyExtension(Extension):

    def extendMarkdown(self, md, md_globals):
        md.registerExtension(self)
        md.preprocessors.add('mentionify',
                             MentionifyPreprocessor(md),
                             '_end')


class MentionifyPreprocessor(Preprocessor):

    def run(self, lines):
        new_lines = []
        matches = set()
        mentions = {}

        def mentionify(match):
            username = match.group(2)

            if len(matches) >= settings.ST_MENTIONS_PER_COMMENT:
                return match.group(0)

            if username in matches:
                return match.group(0)

            matches.add(username)

            try:
                user = User.objects.get(username=username)
            except User.DoesNotExist:
                return match.group(0)

            mentions[username] = user

            return u'%s[@%s](%s)' % (match.group(1), username, user.get_absolute_url())

        for line in lines:
            if line.strip() and not line.startswith('>'):  # exclude code/quote
                line = re.sub(ur'([^\w]?)@(?P<username>[\w.@+-]+)', mentionify, line, flags=re.UNICODE)

            new_lines.append(line)

        # markdown_instance
        self.markdown.mentions = mentions

        return new_lines


def makeExtension(configs=None):
    return MentionifyExtension(configs=configs)


#-*- coding: utf-8 -*-

from . import register
from spirit.models.comment_like import CommentLike
from spirit.forms.comment_like import LikeForm


@register.inclusion_tag('spirit/comment_like/_form.html')
def render_like_form(comment, like, next=None):
    form = LikeForm()
    return {'form': form, 'comment_id': comment.pk, 'like': like, 'next': next}


@register.simple_tag()
def populate_likes(comments, user, to_attr='like'):
    # TODO: use Prefetch on django 1.7, move this to CustomQuerySet.as_manager
    likes = CommentLike.objects.filter(comment__in=comments, user=user)
    likes_dict = {l.comment_id: l for l in likes}

    for c in comments:
        setattr(c, to_attr, likes_dict.get(c.pk))

    return ""


#-*- coding: utf-8 -*-

from django.db.models import Manager


class TopicUnreadManager(Manager):

    def for_user(self, user):
        return self.filter(user=user, is_read=False, is_removed=False)

    def read(self, user, topic):
        # returns updated rows count (int)
        return self.filter(user=user, topic=topic)\
            .update(is_read=True)


#-*- coding: utf-8 -*-

from django.db.models import Manager
from django.db.models import Q
from django.shortcuts import get_object_or_404


class CategoryManager(Manager):

    def for_public(self):
        return self.filter(Q(parent=None) | Q(parent__is_removed=False),
                           is_removed=False,
                           is_private=False)

    def for_public_open(self):
        return self.for_public()\
            .filter(Q(parent=None) | Q(parent__is_closed=False),
                    is_closed=False)

    def for_parent(self, parent=None):
        if parent and parent.is_subcategory:
            return self.none()
        else:
            return self.filter(parent=parent,
                               is_removed=False,
                               is_private=False)

    def get_public_or_404(self, pk):
        return get_object_or_404(self.for_public(),
                                 pk=pk)


#-*- coding: utf-8 -*-

import datetime

from django.test import TestCase
from django.core.urlresolvers import reverse
from django.template import Template, Context, TemplateSyntaxError
from django.core.cache import cache
from django.contrib.auth import get_user_model
from django.core import mail
from django.utils.translation import ugettext as _
from django.utils import timezone

import utils

from spirit.forms.user import RegistrationForm, UserProfileForm, EmailChangeForm, ResendActivationForm
from spirit.backends.user import EmailAuthBackend
from spirit.models.comment_like import CommentLike
from spirit.utils.user.tokens import UserActivationTokenGenerator, UserEmailChangeTokenGenerator
from spirit.models.user import User as UserModel
from spirit.models.topic import Topic
from spirit.models.comment import Comment


User = get_user_model()


class UserViewTest(TestCase):

    fixtures = ['spirit_init.json', ]

    def setUp(self):
        cache.clear()
        self.user = utils.create_user()
        self.user2 = utils.create_user()
        self.category = utils.create_category()
        self.topic = utils.create_topic(self.category, user=self.user2)
        self.topic2 = utils.create_topic(self.category)

    def test_login_email(self):
        """
        try to login by email
        """
        # get
        response = self.client.get(reverse('spirit:user-login'))
        self.assertEqual(response.status_code, 200)

        # post
        form_data = {'username': self.user.email, 'password': "bar"}
        response = self.client.post(reverse('spirit:user-login'),
                                    form_data)
        expected_url = reverse('spirit:profile-update')
        self.assertRedirects(response, expected_url, status_code=302)

    def test_login_redirect(self):
        """
        try to login with a logged in user
        """
        utils.login(self)
        response = self.client.get(reverse('spirit:user-login'))
        expected_url = self.user.get_absolute_url()
        self.assertRedirects(response, expected_url, status_code=302)
        # next
        response = self.client.get(reverse('spirit:user-login') + '?next=/fakepath/')
        self.assertRedirects(response, '/fakepath/', status_code=302, target_status_code=404)

    def test_register(self):
        """
        register
        """
        # get
        response = self.client.get(reverse('spirit:user-register'))
        self.assertEqual(response.status_code, 200)

        # post
        form_data = {'username': 'uniquefoo', 'email': 'some@some.com', 'password1': 'pass', 'password2': 'pass'}
        response = self.client.post(reverse('spirit:user-register'),
                                    form_data)
        expected_url = reverse('spirit:user-login')
        self.assertRedirects(response, expected_url, status_code=302)

        # redirect logged in user
        utils.login(self)
        response = self.client.get(reverse('spirit:user-register'))
        self.assertRedirects(response, reverse('spirit:profile-update'), status_code=302)

    def test_register_email_sent(self):
        """
        register and send activation email
        """
        form_data = {'username': 'uniquefoo', 'email': 'some@some.com', 'password1': 'pass', 'password2': 'pass'}
        response = self.client.post(reverse('spirit:user-register'), form_data)
        self.assertEqual(response.status_code, 302)
        self.assertEquals(len(mail.outbox), 1)
        self.assertEquals(mail.outbox[0].subject, _("User activation"))

    def test_register_next_logged_in(self):
        """
        redirect next on register
        """
        # redirect logged in user
        utils.login(self)
        response = self.client.get(reverse('spirit:user-register') + "?next=/fakepath/")
        self.assertRedirects(response, '/fakepath/', status_code=302, target_status_code=404)

    def test_profile_topics(self):
        """
        profile user's topics
        """
        utils.login(self)
        response = self.client.get(reverse("spirit:profile-topics", kwargs={'pk': self.user2.pk,
                                                                            'slug': self.user2.slug}))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['topics'], [repr(self.topic), ])
        self.assertEqual(repr(response.context['p_user']), repr(self.user2))

    def test_profile_topics_order(self):
        """
        topics ordered by date
        """
        Topic.objects.all().delete()

        category = utils.create_category()
        topic_a = utils.create_topic(category=category, user=self.user2)
        topic_b = utils.create_topic(category=category, user=self.user2)
        topic_c = utils.create_topic(category=category, user=self.user2)

        Topic.objects.filter(pk=topic_a.pk).update(date=timezone.now() - datetime.timedelta(days=10))
        Topic.objects.filter(pk=topic_c.pk).update(date=timezone.now() - datetime.timedelta(days=5))

        utils.login(self)
        response = self.client.get(reverse("spirit:profile-topics", kwargs={'pk': self.user2.pk,
                                                                            'slug': self.user2.slug}))
        self.assertQuerysetEqual(response.context['topics'], map(repr, [topic_b, topic_c, topic_a]))

    def test_profile_topics_dont_show_removed_or_private(self):
        """
        dont show private topics or removed
        """
        Topic.objects.all().delete()

        category = utils.create_category()
        category_removed = utils.create_category(is_removed=True)
        subcategory = utils.create_category(parent=category_removed)
        subcategory_removed = utils.create_category(parent=category, is_removed=True)
        topic_a = utils.create_private_topic(user=self.user2)
        topic_b = utils.create_topic(category=category, user=self.user2, is_removed=True)
        topic_c = utils.create_topic(category=category_removed, user=self.user2)
        topic_d = utils.create_topic(category=subcategory, user=self.user2)
        topic_e = utils.create_topic(category=subcategory_removed, user=self.user2)

        utils.login(self)
        response = self.client.get(reverse("spirit:profile-topics", kwargs={'pk': self.user2.pk,
                                                                            'slug': self.user2.slug}))
        self.assertQuerysetEqual(response.context['topics'], [])

    def test_profile_topics_invalid_slug(self):
        """
        profile user's topics
        """
        utils.login(self)
        response = self.client.get(reverse("spirit:profile-topics", kwargs={'pk': self.user2.pk,
                                                                            'slug': "invalid"}))
        expected_url = reverse("spirit:profile-topics", kwargs={'pk': self.user2.pk,
                                                                'slug': self.user2.slug})
        self.assertRedirects(response, expected_url, status_code=301)

    def test_profile_comments(self):
        """
        profile user's comments
        """
        utils.login(self)
        comment = utils.create_comment(user=self.user2, topic=self.topic)
        comment2 = utils.create_comment(user=self.user, topic=self.topic)
        response = self.client.get(reverse("spirit:profile-detail", kwargs={'pk': self.user2.pk,
                                                                            'slug': self.user2.slug}))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['comments'], [repr(comment), ])
        self.assertEqual(repr(response.context['p_user']), repr(self.user2))

    def test_profile_comments_order(self):
        """
        comments ordered by date
        """
        comment_a = utils.create_comment(user=self.user2, topic=self.topic)
        comment_b = utils.create_comment(user=self.user2, topic=self.topic)
        comment_c = utils.create_comment(user=self.user2, topic=self.topic)

        Comment.objects.filter(pk=comment_a.pk).update(date=timezone.now() - datetime.timedelta(days=10))
        Comment.objects.filter(pk=comment_c.pk).update(date=timezone.now() - datetime.timedelta(days=5))

        utils.login(self)
        response = self.client.get(reverse("spirit:profile-detail", kwargs={'pk': self.user2.pk,
                                                                            'slug': self.user2.slug}))
        self.assertQuerysetEqual(response.context['comments'], map(repr, [comment_b, comment_c, comment_a]))

    def test_profile_comments_dont_show_removed_or_private(self):
        """
        dont show private topics or removed
        """
        category = utils.create_category()
        category_removed = utils.create_category(is_removed=True)
        subcategory = utils.create_category(parent=category_removed)
        subcategory_removed = utils.create_category(parent=category, is_removed=True)
        topic_a = utils.create_private_topic(user=self.user2)
        topic_b = utils.create_topic(category=category, is_removed=True)
        topic_c = utils.create_topic(category=category_removed)
        topic_d = utils.create_topic(category=subcategory)
        topic_e = utils.create_topic(category=subcategory_removed)
        comment_a = utils.create_comment(user=self.user2, topic=topic_a.topic)
        comment_b = utils.create_comment(user=self.user2, topic=topic_b)
        comment_c = utils.create_comment(user=self.user2, topic=topic_c)
        comment_d = utils.create_comment(user=self.user2, topic=topic_d)
        comment_e = utils.create_comment(user=self.user2, topic=topic_e)

        utils.login(self)
        response = self.client.get(reverse("spirit:profile-detail", kwargs={'pk': self.user2.pk,
                                                                            'slug': self.user2.slug}))
        self.assertQuerysetEqual(response.context['comments'], [])

    def test_profile_comments_invalid_slug(self):
        """
        profile user's comments, invalid user slug
        """
        utils.login(self)
        response = self.client.get(reverse("spirit:profile-detail", kwargs={'pk': self.user2.pk,
                                                                            'slug': "invalid"}))
        expected_url = reverse("spirit:profile-detail", kwargs={'pk': self.user2.pk,
                                                                'slug': self.user2.slug})
        self.assertRedirects(response, expected_url, status_code=301)

    def test_profile_likes(self):
        """
        profile user's likes
        """
        utils.login(self)
        comment = utils.create_comment(user=self.user, topic=self.topic)
        comment2 = utils.create_comment(user=self.user2, topic=self.topic)
        like = CommentLike.objects.create(user=self.user2, comment=comment)
        like2 = CommentLike.objects.create(user=self.user, comment=comment2)
        response = self.client.get(reverse("spirit:profile-likes", kwargs={'pk': self.user2.pk,
                                                                           'slug': self.user2.slug}))
        self.assertEqual(response.status_code, 200)
        self.assertQuerysetEqual(response.context['comments'], [repr(like.comment), ])
        self.assertEqual(repr(response.context['p_user']), repr(self.user2))

    def test_profile_likes_order(self):
        """
        comments ordered by date
        """
        comment_a = utils.create_comment(user=self.user, topic=self.topic)
        comment_b = utils.create_comment(user=self.user, topic=self.topic)
        comment_c = utils.create_comment(user=self.user, topic=self.topic)
        like_a = CommentLike.objects.create(user=self.user2, comment=comment_a)
        like_b = CommentLike.objects.create(user=self.user2, comment=comment_b)
        like_c = CommentLike.objects.create(user=self.user2, comment=comment_c)

        CommentLike.objects.filter(pk=like_a.pk).update(date=timezone.now() - datetime.timedelta(days=10))
        CommentLike.objects.filter(pk=like_c.pk).update(date=timezone.now() - datetime.timedelta(days=5))

        utils.login(self)
        response = self.client.get(reverse("spirit:profile-likes", kwargs={'pk': self.user2.pk,
                                                                           'slug': self.user2.slug}))
        self.assertQuerysetEqual(response.context['comments'], map(repr, [comment_b, comment_c, comment_a]))

    def test_profile_likes_dont_show_removed_or_private(self):
        """
        dont show private topics or removed
        """
        category = utils.create_category()
        category_removed = utils.create_category(is_removed=True)
        subcategory = utils.create_category(parent=category_removed)
        subcategory_removed = utils.create_category(parent=category, is_removed=True)
        topic_a = utils.create_private_topic(user=self.user2)
        topic_b = utils.create_topic(category=category, is_removed=True)
        topic_c = utils.create_topic(category=category_removed)
        topic_d = utils.create_topic(category=subcategory)
        topic_e = utils.create_topic(category=subcategory_removed)
        comment_a = utils.create_comment(user=self.user2, topic=topic_a.topic)
        comment_b = utils.create_comment(user=self.user, topic=topic_b)
        comment_c = utils.create_comment(user=self.user, topic=topic_c)
        comment_d = utils.create_comment(user=self.user, topic=topic_d)
        comment_e = utils.create_comment(user=self.user, topic=topic_e)
        like_a = CommentLike.objects.create(user=self.user2, comment=comment_a)
        like_b = CommentLike.objects.create(user=self.user2, comment=comment_b)
        like_c = CommentLike.objects.create(user=self.user2, comment=comment_c)
        like_d = CommentLike.objects.create(user=self.user2, comment=comment_d)
        like_e = CommentLike.objects.create(user=self.user2, comment=comment_e)

        utils.login(self)
        response = self.client.get(reverse("spirit:profile-likes", kwargs={'pk': self.user2.pk,
                                                                            'slug': self.user2.slug}))
        self.assertQuerysetEqual(response.context['comments'], [])

    def test_profile_likes_invalid_slug(self):
        """
        profile user's likes, invalid user slug
        """
        utils.login(self)
        response = self.client.get(reverse("spirit:profile-likes", kwargs={'pk': self.user2.pk,
                                                                           'slug': "invalid"}))
        expected_url = reverse("spirit:profile-likes", kwargs={'pk': self.user2.pk,
                                                               'slug': self.user2.slug})
        self.assertRedirects(response, expected_url, status_code=301)

    def test_profile_update(self):
        """
        profile update
        """
        utils.login(self)
        # get
        response = self.client.get(reverse('spirit:profile-update'))
        self.assertEqual(response.status_code, 200)

        # post
        form_data = {'first_name': 'foo', 'last_name': 'bar',
                     'location': 'spirit', 'timezone': self.user.timezone}
        response = self.client.post(reverse('spirit:profile-update'),
                                    form_data)
        expected_url = reverse('spirit:profile-update')
        self.assertRedirects(response, expected_url, status_code=302)

    def test_login_rate_limit(self):
        """
        test rate limit 5/5m
        """
        form_data = {'username': self.user.email, 'password': "badpassword"}
        url = reverse('spirit:user-login') + "?next=/path/"
        for _ in xrange(6):
            response = self.client.post(url, form_data)
        self.assertRedirects(response, url, status_code=302)

    def test_custom_reset_password(self):
        """
        test rate limit 5/5m
        """
        form_data = {'email': "bademail@bad.com", }
        for _ in xrange(6):
            response = self.client.post(reverse('spirit:password-reset'),
                                        form_data)
        expected_url = reverse("spirit:password-reset")
        self.assertRedirects(response, expected_url, status_code=302)

    def test_password_reset_confirm(self):
        """
        test access
        """
        response = self.client.get(reverse('spirit:password-reset-confirm', kwargs={'uidb64': 'f-a-k-e',
                                                                                    'token': 'f-a-k-e'}))
        self.assertEqual(response.status_code, 200)

    def test_admin_login(self):
        """
        redirect to regular user login
        if fails, make sure you added:
        admin.site.login = login_required(admin.site.login)
        to urls.py (the one in your project's root)
        """
        response = self.client.get(reverse('admin:index'))
        expected_url = reverse("spirit:user-login") + "?next=" + reverse('admin:index')
        self.assertRedirects(response, expected_url, status_code=302)

    def test_profile_password_change(self):
        """
        profile password change
        """
        user = utils.create_user(password="foo")
        utils.login(self, user=user, password="foo")
        form_data = {'old_password': 'foo',
                     'new_password1': 'bar',
                     'new_password2': 'bar'}
        response = self.client.post(reverse('spirit:profile-password-change'),
                                    form_data)
        expected_url = reverse("spirit:profile-update")
        self.assertRedirects(response, expected_url, status_code=302)
        utils.login(self, user=user, password="bar")

        # get
        response = self.client.get(reverse('spirit:profile-password-change'))
        self.assertEqual(response.status_code, 200)

    def test_registration_activation(self):
        """
        registration activation
        """
        self.user.is_active = False
        self.user.save()
        token = UserActivationTokenGenerator().generate(self.user)
        response = self.client.get(reverse('spirit:registration-activation', kwargs={'pk': self.user.pk,
                                                                                     'token': token}))
        expected_url = reverse("spirit:user-login")
        self.assertRedirects(response, expected_url, status_code=302)
        self.assertTrue(User.objects.get(pk=self.user.pk).is_active)

    def test_registration_activation_invalid(self):
        """
        Activation token should expire after first login
        ActiveUserMiddleware required
        """
        self.user.last_login = self.user.last_login - datetime.timedelta(hours=1)
        token = UserActivationTokenGenerator().generate(self.user)

        utils.login(self)
        User.objects.filter(pk=self.user.pk).update(is_active=False)
        response = self.client.get(reverse('spirit:registration-activation', kwargs={'pk': self.user.pk,
                                                                                     'token': token}))
        expected_url = reverse("spirit:user-login")
        self.assertRedirects(response, expected_url, status_code=302)
        self.assertFalse(User.objects.get(pk=self.user.pk).is_active)

    def test_email_change_confirm(self):
        """
        email change confirmation
        """
        utils.login(self)
        new_email = "newfoo@bar.com"
        token = UserEmailChangeTokenGenerator().generate(self.user, new_email)
        response = self.client.get(reverse('spirit:email-change-confirm', kwargs={'token': token}))
        expected_url = reverse("spirit:profile-update")
        self.assertRedirects(response, expected_url, status_code=302)
        self.assertEqual(User.objects.get(pk=self.user.pk).email, new_email)

    def test_email_change_confirm_invalid(self):
        """
        The token should expire after email change
        """
        utils.login(self)
        old_email = "oldfoo@bar.com"
        token = UserEmailChangeTokenGenerator().generate(self.user, old_email)
        new_email = "newfoo@bar.com"
        User.objects.filter(pk=self.user.pk).update(email=new_email)
        response = self.client.get(reverse('spirit:email-change-confirm', kwargs={'token': token}))
        expected_url = reverse("spirit:profile-update")
        self.assertRedirects(response, expected_url, status_code=302)
        self.assertEqual(User.objects.get(pk=self.user.pk).email, new_email)

    def test_profile_email_change(self):
        """
        email change
        """
        user = utils.create_user(password="foo")
        utils.login(self, user=user, password="foo")
        form_data = {'password': 'foo',
                     'email': 'newfoo@bar.com'}
        response = self.client.post(reverse('spirit:profile-email-change'),
                                    form_data)
        expected_url = reverse("spirit:profile-update")
        self.assertRedirects(response, expected_url, status_code=302)
        self.assertEquals(len(mail.outbox), 1)
        self.assertIn(_("Email change"), mail.outbox[0].subject)

        # get
        response = self.client.get(reverse('spirit:profile-email-change'))
        self.assertEqual(response.status_code, 200)

    def test_resend_activation_email(self):
        """
        resend_activation_email
        """
        user = utils.create_user(password="foo")

        form_data = {'email': user.email,
                     'password': "foo"}
        response = self.client.post(reverse('spirit:resend-activation'),
                                    form_data)
        expected_url = reverse("spirit:user-login")
        self.assertRedirects(response, expected_url, status_code=302)
        self.assertEquals(len(mail.outbox), 1)
        self.assertEquals(mail.outbox[0].subject, _("User activation"))

        # get
        response = self.client.get(reverse('spirit:resend-activation'))
        self.assertEquals(response.status_code, 200)

    def test_resend_activation_email_invalid_previously_logged_in(self):
        """
        resend_activation_email invalid if last_ip was set
        """
        user = utils.create_user(password="foo", last_ip="1.1.1.1")

        form_data = {'email': user.email,
                     'password': "foo"}
        response = self.client.post(reverse('spirit:resend-activation'),
                                    form_data)
        self.assertEquals(response.status_code, 302)
        self.assertEquals(len(mail.outbox), 0)

    def test_resend_activation_email_invalid_email(self):
        """
        resend_activation_email invalid password
        """
        user = utils.create_user(password="foo")

        form_data = {'email': "bad@foo.com", }
        response = self.client.post(reverse('spirit:resend-activation'),
                                    form_data)
        self.assertEquals(response.status_code, 302)
        self.assertEquals(len(mail.outbox), 0)

    def test_resend_activation_email_redirect_logged(self):
        """
        resend_activation_email redirect to profile if user is logged in
        """
        utils.login(self)
        response = self.client.get(reverse('spirit:resend-activation'))
        expected_url = reverse("spirit:profile-update")
        self.assertRedirects(response, expected_url, status_code=302)

    def test_logout(self):
        """
        should log out on POST only
        """
        utils.login(self)

        # get should display confirmation message
        response = self.client.get(reverse('spirit:user-logout'))
        self.assertEqual(response.status_code, 200)
        self.assertTrue(self.client.session.items())

        # post should log out the user (clear the session)
        response = self.client.post(reverse('spirit:user-logout'))
        expected_url = "/"
        self.assertRedirects(response, expected_url, status_code=302)
        self.assertFalse(self.client.session.items())

        # next
        utils.login(self)
        self.assertTrue(self.client.session.items())
        response = self.client.post(reverse('spirit:user-logout') + '?next=/fakepath/')
        self.assertRedirects(response, '/fakepath/', status_code=302, target_status_code=404)
        self.assertFalse(self.client.session.items())

    def test_logout_anonymous_redirect(self):
        """
        should log out on POST only
        """
        # redirect to login if user is anonymous
        response = self.client.get(reverse('spirit:user-logout'))
        expected_url = reverse("spirit:user-login")
        self.assertRedirects(response, expected_url, status_code=302)

        # next if user is anonymous
        response = self.client.get(reverse('spirit:user-logout') + '?next=/fakepath/')
        self.assertRedirects(response, '/fakepath/', status_code=302, target_status_code=404)


class UserFormTest(TestCase):

    fixtures = ['spirit_init.json', ]

    def setUp(self):
        cache.clear()
        self.user = utils.create_user()

    def test_registration(self):
        """
        register
        """
        form_data = {'username': 'foo', 'email': 'foo@foo.com',
                     'password1': 'pass', 'password2': 'pass'}
        form = RegistrationForm(data=form_data)
        self.assertEqual(form.is_valid(), True)

    def test_registration_invalid(self):
        """
        invalid email and user
        """
        User.objects.create_user(username="foo", password="bar", email="foo@foo.com")
        form_data = {'username': 'foo', 'email': 'foo@foo.com',
                     'password1': 'pass', 'password2': 'pass'}
        form = RegistrationForm(data=form_data)
        self.assertEqual(form.is_valid(), False)
        self.assertNotIn('username', form.cleaned_data)
        self.assertNotIn('foo@foo.com', form.cleaned_data)

    def test_registration_honeypot(self):
        """
        registration honeypot
        """
        form_data = {'username': 'foo', 'email': 'foo@foo.com',
                     'password1': 'pass', 'password2': 'pass',
                     'honeypot': 'im a robot'}
        form = RegistrationForm(data=form_data)
        self.assertEqual(form.is_valid(), False)
        self.assertNotIn('honeypot', form.cleaned_data)

    def test_profile(self):
        """
        edit user profile
        """
        form_data = {'first_name': 'foo', 'last_name': 'bar',
                     'location': 'spirit', 'timezone': self.user.timezone}
        form = UserProfileForm(data=form_data, instance=self.user)
        self.assertEqual(form.is_valid(), True)

    def test_email_change(self):
        """
        email change
        """
        user = utils.create_user(password="foo")
        form_data = {'email': 'newfoo@bar.com', 'password': 'foo'}
        form = EmailChangeForm(data=form_data, user=user)
        self.assertEqual(form.is_valid(), True)

    def test_email_change_invalid(self):
        """
        email change invalid
        """
        user = utils.create_user(password="foo", email="newfoo@bar.com")
        form_data = {'email': 'newfoo@bar.com', 'password': 'bad-password'}
        form = EmailChangeForm(data=form_data, user=user)
        self.assertEqual(form.is_valid(), False)
        self.assertNotIn('email', form.cleaned_data)
        self.assertNotIn('password', form.cleaned_data)

    def test_resend_activation_email(self):
        """
        resend activation
        """
        user = utils.create_user(email="newfoo@bar.com")
        form_data = {'email': 'newfoo@bar.com', }
        form = ResendActivationForm(form_data)
        self.assertTrue(form.is_valid())
        self.assertEqual(form.get_user(), user)

    def test_resend_activation_email_invalid_email(self):
        """
        resend activation invalid
        """
        form_data = {'email': 'bad@bar.com', }
        form = ResendActivationForm(form_data)
        self.assertFalse(form.is_valid())


class UserBackendTest(TestCase):

    fixtures = ['spirit_init.json', ]

    def setUp(self):
        cache.clear()
        self.user = utils.create_user(email="foobar@bar.com", password="bar")

    def test_email_auth_backend(self):
        user = EmailAuthBackend().authenticate(username="foobar@bar.com", password="bar")
        self.assertEqual(user, self.user)


class UserModelTest(TestCase):

    fixtures = ['spirit_init.json', ]

    def setUp(self):
        cache.clear()

    def test_user_superuser(self):
        """
        is_superuser should always be is_administrator and is_moderator
        test model
        """
        user = UserModel(is_superuser=True)
        user.save()
        self.assertTrue(user.is_administrator)
        self.assertTrue(user.is_moderator)

    def test_user_administrator(self):
        """
        is_administrator should always be is_moderator
        """
        user = UserModel(is_administrator=True)
        user.save()
        self.assertTrue(user.is_moderator)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from setuptools import setup, find_packages

README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()
REQUIREMENTS = open(os.path.join(os.path.dirname(__file__), 'requirements.txt')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-spirit',
    version='0.1.2',
    description='Spirit is a Python based forum powered by Django.',
    author='Esteban Castro Borsani',
    author_email='ecastroborsani@gmail.com',
    long_description=README,
    url='http://spirit-project.com/',
    packages=find_packages(exclude=['example', ]),
    test_suite="run_tests.run_tests",
    include_package_data=True,
    zip_safe=False,
    install_requires=REQUIREMENTS,
    license='MIT License',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



#-*- coding: utf-8 -*-

from collections import OrderedDict

from django.contrib.messages import constants

from .. import register


TAGS = {
    constants.DEBUG: 'debug',
    constants.INFO: 'info',
    constants.SUCCESS: 'success',
    constants.WARNING: 'warning',
    constants.ERROR: 'error',
}


@register.inclusion_tag('spirit/utils/_messages.html')
def render_messages(messages):
    grouped = OrderedDict()

    for m in messages:
        messages_group = grouped.get(TAGS[m.level], [])
        messages_group.append(m)
        grouped[TAGS[m.level]] = messages_group

    return {'messages_grouped': grouped, }


#-*- coding: utf-8 -*-

from djconfig.forms import ConfigForm

from django import forms
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model

from spirit.models.category import Category
from spirit.models.comment_flag import CommentFlag


User = get_user_model()


class UserEditForm(forms.ModelForm):

    class Meta:
        model = User
        fields = ("username", "email", "location",
                  "timezone", "is_administrator", "is_moderator", "is_active")


class CategoryForm(forms.ModelForm):

    class Meta:
        model = Category
        fields = ("parent", "title", "description", "is_closed", "is_removed")
    
    def __init__(self, *args, **kwargs):
        super(CategoryForm, self).__init__(*args, **kwargs)
        queryset = Category.objects.for_parent()

        if self.instance.pk:
            queryset = queryset.exclude(pk=self.instance.pk)

        self.fields['parent'] = forms.ModelChoiceField(queryset=queryset, required=False)

    def clean_parent(self):
        parent = self.cleaned_data["parent"]

        if self.instance.pk:
            has_childrens = self.instance.category_set.all().exists()

            if parent and has_childrens:
                raise forms.ValidationError(_("The category you are updating "
                                              "can not have a parent since it has childrens"))

        return parent


class CommentFlagForm(forms.ModelForm):

    class Meta:
        model = CommentFlag
        fields = ("is_closed", )

    def __init__(self, user=None, *args, **kwargs):
        super(CommentFlagForm, self).__init__(*args, **kwargs)
        self.user = user

    def save(self, commit=True):
        self.instance.moderator = self.user
        return super(CommentFlagForm, self).save(commit)


class BasicConfigForm(ConfigForm):

    site_name = forms.CharField(initial="Spirit", label=_("site name"))
    site_description = forms.CharField(initial="", label=_("site description"), max_length=75, required=False)
    template_footer = forms.CharField(initial="", label=_("footer snippet"), required=False,
                                      widget=forms.Textarea(attrs={'rows': 2, }),
                                      help_text=_("This gets rendered just before the footer in your template."))


#-*- coding: utf-8 -*-

from django.contrib.auth.decorators import login_required
from django.shortcuts import render, redirect, get_object_or_404
from django.http import HttpResponsePermanentRedirect
from django.conf import settings

from spirit.utils.ratelimit.decorators import ratelimit
from spirit.utils.decorators import moderator_required
from spirit.models.category import Category
from spirit.models.comment import MOVED, CLOSED, UNCLOSED, PINNED, UNPINNED
from spirit.forms.comment import CommentForm
from spirit.signals.comment import comment_posted

from spirit.models.topic import Topic
from spirit.forms.topic import TopicForm
from spirit.signals.topic import topic_viewed, topic_post_moderate


@login_required
@ratelimit(rate='1/10s')
def topic_publish(request, category_id=None):
    if category_id:
        Category.objects.get_public_or_404(pk=category_id)

    if request.method == 'POST':
        form = TopicForm(user=request.user, data=request.POST)
        cform = CommentForm(user=request.user, data=request.POST)

        if not request.is_limited and form.is_valid() and cform.is_valid():
            # wrap in transaction.atomic?
            topic = form.save()
            cform.topic = topic
            comment = cform.save()
            comment_posted.send(sender=comment.__class__, comment=comment, mentions=cform.mentions)
            return redirect(topic.get_absolute_url())
    else:
        form = TopicForm(user=request.user, initial={'category': category_id, })
        cform = CommentForm()

    return render(request, 'spirit/topic/topic_publish.html', {'form': form, 'cform': cform})


@login_required
def topic_update(request, pk):
    topic = Topic.objects.for_update_or_404(pk, request.user)

    if request.method == 'POST':
        form = TopicForm(user=request.user, data=request.POST, instance=topic)
        category_id = topic.category_id

        if form.is_valid():
            topic = form.save()

            if topic.category_id != category_id:
                topic_post_moderate.send(sender=topic.__class__, user=request.user, topic=topic, action=MOVED)

            return redirect(request.POST.get('next', topic.get_absolute_url()))
    else:
        form = TopicForm(user=request.user, instance=topic)

    return render(request, 'spirit/topic/topic_update.html', {'form': form, })


def topic_detail(request, pk, slug):
    topic = Topic.objects.get_public_or_404(pk, request.user)

    if topic.slug != slug:
        return HttpResponsePermanentRedirect(topic.get_absolute_url())

    topic_viewed.send(sender=topic.__class__, request=request, topic=topic)

    return render(request, 'spirit/topic/topic_detail.html', {'topic': topic,
                                                              'COMMENTS_PER_PAGE': settings.ST_COMMENTS_PER_PAGE})


@moderator_required
def topic_moderate(request, pk, value, remove=False, lock=False, pin=False):
    # TODO: move to topic_moderate and split it in many views
    # delete TopicUnread on remove
    topic = get_object_or_404(Topic, pk=pk)

    if request.method == 'POST':
        not_value = not value

        if remove:
            Topic.objects.filter(pk=pk, is_removed=not_value)\
                .update(is_removed=value)

        if lock:
            count = Topic.objects.filter(pk=pk, is_closed=not_value)\
                .update(is_closed=value)

            if count:
                action = CLOSED if value else UNCLOSED
                topic_post_moderate.send(sender=topic.__class__, user=request.user, topic=topic, action=action)

        if pin:
            count = Topic.objects.filter(pk=pk, is_pinned=not_value)\
                .update(is_pinned=value)

            if count:
                action = PINNED if value else UNPINNED
                topic_post_moderate.send(sender=topic.__class__, user=request.user, topic=topic, action=action)

        return redirect(request.POST.get('next', topic.get_absolute_url()))

    return render(request, 'spirit/topic/topic_moderate.html', {'topic': topic, })


def topics_active(request):
    topics = Topic.objects.for_public().filter(is_pinned=False)
    topics_pinned = Topic.objects.filter(category_id=settings.ST_UNCATEGORIZED_CATEGORY_PK,
                                         is_removed=False,
                                         is_pinned=True)
    topics = topics | topics_pinned
    topics = topics.order_by('-is_pinned', '-last_active').select_related('category')
    categories = Category.objects.for_parent()

    return render(request, 'spirit/topic/topics_active.html', {'categories': categories,
                                                               'topics': topics})


#-*- coding: utf-8 -*-

import re

from markdown import Extension
from markdown.preprocessors import Preprocessor


# Try to get the video ID. Works for URLs of the form:
# * https://vimeo.com/11111111
# * https://www.vimeo.com/11111111
# * https://player.vimeo.com/video/11111111
# * https://vimeo.com/channels/11111111
# * https://vimeo.com/groups/name/videos/11111111
# * https://vimeo.com/album/2222222/video/11111111
# * https://vimeo.com/11111111?param=value
PATTERN_RE = ur'^https?://(www\.|player\.)?vimeo\.com/(channels/|groups/[^/]+/videos/|album/(\d+)/video/|video/)?' \
             ur'(?P<id>\d+)(\?[^\s]+)?$'


class VimeofyExtension(Extension):

    def extendMarkdown(self, md, md_globals):
        md.registerExtension(self)
        md.preprocessors.add('vimeofy',
                             VimeofyPreprocessor(md),
                             '_end')


class VimeofyPreprocessor(Preprocessor):

    def run(self, lines):
        new_lines = []

        def vimeofy(match):
            video_id = match.group("id")
            html = u'<span class="video"><iframe src="https://player.vimeo.com/video/{video_id}" ' \
                   u'allowfullscreen></iframe></span>'.format(video_id=video_id)
            return self.markdown.htmlStash.store(html, safe=True)

        for line in lines:
            if line.strip():
                line = re.sub(PATTERN_RE, vimeofy, line, flags=re.UNICODE)

            new_lines.append(line)

        return new_lines


def makeExtension(configs=None):
    return VimeofyExtension(configs=configs)


#-*- coding: utf-8 -*-

from django.db.models import Manager


class CommentLikeManager(Manager):

    def for_user(self, user):
        return self.filter(user=user)\
            .select_related('user', 'comment__user')